@@ -12,16 +12,12 @@ GQuark r_install_error_quark(void);
 
 typedef enum {
 	R_INSTALL_ERROR_FAILED,
-	R_INSTALL_ERROR_NOSRC,
-	R_INSTALL_ERROR_NODST,
 	R_INSTALL_ERROR_COMPAT_MISMATCH,
+	R_INSTALL_ERROR_VERSION_MISMATCH,
 	R_INSTALL_ERROR_REJECTED,
 	R_INSTALL_ERROR_MARK_BOOTABLE,
 	R_INSTALL_ERROR_MARK_NONBOOTABLE,
 	R_INSTALL_ERROR_TARGET_GROUP,
-	R_INSTALL_ERROR_DOWNLOAD_MF,
-	R_INSTALL_ERROR_HANDLER,
-	R_INSTALL_ERROR_NO_SUPPORTED,
 	R_INSTALL_ERROR_MOUNTED,
 } RInstallError;
 
@@ -34,6 +30,7 @@ typedef struct {
 	gint status_result;
 	/* install options */
 	gboolean ignore_compatible;
+	gboolean ignore_version_limit;
 	gchar *transaction;
 	RaucBundleAccessArgs access_args;
 } RaucInstallArgs;
