@@ -11,6 +11,7 @@ typedef enum {
 	R_UTILS_ERROR_FAILED,
 	R_UTILS_ERROR_INAPPROPRIATE_IOCTL,
 	R_UTILS_ERROR_INVALID_ENV_KEY,
+	R_UTILS_ERROR_SEMVER_PARSE,
 } RUtilsError;
 
 #define BIT(nr) (1UL << (nr))
@@ -82,16 +83,53 @@ G_GNUC_WARN_UNUSED_RESULT;
  * Adds elements of a zero-terminated GStrv/gchar** to an existing GPtrArray
  *
  * @param ptrarray GPtrArray to add to
- * @param argvp arguments to add
+ * @param argvp arguments to add (may be NULL)
  * @param copy whether to just add the pointer (FALSE) or copy the underlying data (TRUE)
  */
 static inline void r_ptr_array_addv(GPtrArray *ptrarray, gchar **argvp, gboolean copy)
 {
+	if (argvp == NULL)
+		return;
+
 	for (gchar **addarg = argvp; *addarg != NULL; addarg++) {
 		g_ptr_array_add(ptrarray, copy ? g_strdup(*addarg) : *addarg);
 	}
 }
 
+/**
+ * Adds a formatted string to the end of a GPtrArray
+ *
+ * This is a shorter alternative to:
+ * g_ptr_array_add(arr, g_strdup_printf("%s: %s", ...));
+ *
+ * @param ptrarray GPtrArray to add to
+ * @param format the printf-like format string
+ * @param ... the parameters for the format string
+ */
+void r_ptr_array_add_printf(GPtrArray *ptrarray, const gchar *format, ...)
+__attribute__((__format__(__printf__, 2, 3)));
+
+/**
+ * Converts an array of 'key=value' strings to a shell quoted string.
+ *
+ * This is useful when generating shell-parsable output.
+ *
+ * @param ptrarray the GPtrArray to print
+ */
+gchar *r_ptr_array_env_to_shell(const GPtrArray *ptrarray);
+
+/**
+ * Calls g_environ_setenv for each 'key=value' string in the array.
+ *
+ * This is useful when setting up the environment for a subprocess.
+ *
+ * @param envp an environment list
+ * @param ptrarray the GPtrArray to add to the environment
+ * @param overwrite whether to change existing variables
+ */
+gchar **r_environ_setenv_ptr_array(gchar **envp, const GPtrArray *ptrarray, gboolean overwrite)
+G_GNUC_WARN_UNUSED_RESULT;
+
 /**
  * Read file content into a GBytes.
  *
@@ -271,6 +309,16 @@ G_GNUC_WARN_UNUSED_RESULT;
 goffset get_device_size(gint fd, GError **error)
 G_GNUC_WARN_UNUSED_RESULT;
 
+/**
+ * Replaces a string pointer with a newly allocated copy of the source string.
+ *
+ * If the pointer was non-NULL previously, the old string is freed.
+ *
+ * @param dst the pointer to update
+ * @param src the string to copy
+ */
+void r_replace_strdup(gchar **dst, const gchar *src);
+
 /**
  * Converts a key for use in an environment variable name.
  *
@@ -340,3 +388,41 @@ void r_fakeroot_add_args(GPtrArray *args, const gchar *envpath);
  */
 gboolean r_fakeroot_cleanup(const gchar *envpath, GError **error)
 G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * Returns the contents of the GBytes as a '\0'-terminated string.
+ *
+ * The provided GBytes pointer is freed and nulled.
+ * Internally, it uses g_strndup.
+ *
+ * @param bytes GBytes to take the contents from
+ *
+ * @return null-terminated string, to be freed by the caller
+ */
+gchar *r_bytes_unref_to_string(GBytes **bytes)
+G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * Parse a "semantic version" string into its constituents.
+ *
+ *
+ * @param version_string input string
+ * @param[out] version_core return location for version-core as {major,minor,patch}
+ * @param[out] pre_release return location for pre_release version part, can be NULL
+ * @param[out] build return location for build version part, can be NULL
+ * @param[out] error return location for a GError, or NULL
+ *
+ * @return TRUE if the parsing was successful, FALSE otherwise
+ */
+gboolean r_semver_parse(const gchar *version_string, guint64 version_core[3], gchar **pre_release, gchar **build, GError **error);
+
+/**
+ * Compare two "semantic version" strings over their version-core and pre_release identifier.
+ *
+ * @param version_string_a version A
+ * @param version_string_b version B
+ * @param error return location for a GError, or NULL
+ *
+ * @return TRUE if A<=B, FALSE otherwise
+ */
+gboolean r_semver_less_equal(const gchar *version_string_a, const gchar *version_string_b, GError **error);
