@@ -10,6 +10,7 @@
 #include <sys/socket.h>
 
 #include <openssl/rand.h>
+#include <unistd.h>
 
 #include "bundle.h"
 #include "context.h"
@@ -63,6 +64,9 @@
 #ifndef NTFS_SB_MAGIC
 #define NTFS_SB_MAGIC 0x5346544e
 #endif
+#ifndef NTFS3_SUPER_MAGIC
+#define NTFS3_SUPER_MAGIC 0x7366746e
+#endif
 #ifndef OVERLAYFS_SUPER_MAGIC
 #define OVERLAYFS_SUPER_MAGIC 0x794c7630
 #endif
@@ -198,9 +202,9 @@ static gboolean casync_make_arch(const gchar *idxpath, const gchar *contentpath,
 {
 	GError *ierror = NULL;
 	gboolean res = FALSE;
-	GPtrArray *args = g_ptr_array_new_full(15, g_free);
-	GPtrArray *iargs = g_ptr_array_new_full(15, g_free);
-	const gchar *tmpdir = NULL;
+	g_autoptr(GPtrArray) args = g_ptr_array_new_full(5, g_free);
+	g_autoptr(GPtrArray) iargs = g_ptr_array_new_full(15, g_free);
+	g_autofree gchar *tmpdir = NULL;
 
 	g_return_val_if_fail(idxpath != NULL, FALSE);
 	g_return_val_if_fail(contentpath != NULL, FALSE);
@@ -255,7 +259,7 @@ static gboolean casync_make_arch(const gchar *idxpath, const gchar *contentpath,
 	g_ptr_array_add(args, g_strdup("fakeroot"));
 	g_ptr_array_add(args, g_strdup("sh"));
 	g_ptr_array_add(args, g_strdup("-c"));
-	g_ptr_array_add(args, g_strjoinv(" ", (gchar**) g_ptr_array_free(iargs, FALSE)));
+	g_ptr_array_add(args, g_strjoinv(" ", (gchar**) iargs->pdata));
 	g_ptr_array_add(args, NULL);
 
 	res = r_subprocess_runv(args, G_SUBPROCESS_FLAGS_STDOUT_SILENCE, &ierror);
@@ -276,7 +280,7 @@ static gboolean casync_make_blob(const gchar *idxpath, const gchar *contentpath,
 {
 	GError *ierror = NULL;
 	gboolean res = FALSE;
-	GPtrArray *args = g_ptr_array_new_full(5, g_free);
+	g_autoptr(GPtrArray) args = g_ptr_array_new_full(5, g_free);
 
 	g_return_val_if_fail(idxpath != NULL, FALSE);
 	g_return_val_if_fail(contentpath != NULL, FALSE);
@@ -452,13 +456,14 @@ static gboolean output_stream_write_uint64_all(GOutputStream *stream,
 		GError **error)
 {
 	gsize bytes_written;
-	gboolean res;
 
 	data = GUINT64_TO_BE(data);
-	res = g_output_stream_write_all(stream, &data, sizeof(data), &bytes_written,
-			cancellable, error);
+	if (!g_output_stream_write_all(stream, &data, sizeof(data), &bytes_written,
+			cancellable, error))
+		return FALSE;
+
 	g_assert(bytes_written == sizeof(data));
-	return res;
+	return TRUE;
 }
 
 static gboolean input_stream_read_uint64_all(GInputStream *stream,
@@ -468,13 +473,14 @@ static gboolean input_stream_read_uint64_all(GInputStream *stream,
 {
 	guint64 tmp;
 	gsize bytes_read;
-	gboolean res;
 
-	res = g_input_stream_read_all(stream, &tmp, sizeof(tmp), &bytes_read,
-			cancellable, error);
+	if (!g_input_stream_read_all(stream, &tmp, sizeof(tmp), &bytes_read,
+			cancellable, error))
+		return FALSE;
+
 	g_assert(bytes_read == sizeof(tmp));
 	*data = GUINT64_FROM_BE(tmp);
-	return res;
+	return TRUE;
 }
 
 static gboolean output_stream_write_bytes_all(GOutputStream *stream,
@@ -498,17 +504,16 @@ static gboolean input_stream_read_bytes_all(GInputStream *stream,
 {
 	g_autofree void *buffer = NULL;
 	gsize bytes_read;
-	gboolean res;
 
 	g_assert_cmpint(count, !=, 0);
 
 	buffer = g_malloc0(count);
 
-	res = g_input_stream_read_all(stream, buffer, count, &bytes_read,
-			cancellable, error);
-	if (!res) {
-		return res;
+	if (!g_input_stream_read_all(stream, buffer, count, &bytes_read,
+			cancellable, error)) {
+		return FALSE;
 	}
+
 	g_assert(bytes_read == count);
 	*bytes = g_bytes_new_take(g_steal_pointer(&buffer), count);
 	return TRUE;
@@ -878,11 +883,95 @@ out:
 	return res;
 }
 
+static gchar *prepare_workdir(const gchar *contentdir, GError **error)
+{
+	GError *ierror = NULL;
+	g_autofree gchar* workdir = NULL;
+	g_autoptr(GDir) dir = NULL;
+	const gchar *name;
+
+	g_return_val_if_fail(contentdir, NULL);
+	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
+
+	/* This could be simplified with g_canonicalize_file() after bumping the
+	 * glib requirement to 2.58. */
+	if (g_path_is_absolute(contentdir)) {
+		workdir = g_build_filename(contentdir, ".rauc-workdir", NULL);
+	} else {
+		g_autofree gchar* cwd = g_get_current_dir();
+		workdir = g_build_filename(cwd, contentdir, ".rauc-workdir", NULL);
+	}
+
+	if (g_file_test(workdir, G_FILE_TEST_EXISTS)) {
+		if (!rm_tree(workdir, &ierror)) {
+			g_propagate_prefixed_error(error, ierror, "Failed to remove old workdir: ");
+			return NULL;
+		}
+	}
+
+	if (g_mkdir(workdir, 0700) != 0) {
+		int err = errno;
+		g_set_error(error, G_FILE_ERROR, g_file_error_from_errno(err),
+				"failed to create the workdir '%s': %s", workdir, g_strerror(err));
+		return NULL;
+	}
+
+	dir = g_dir_open(contentdir, 0, &ierror);
+	if (!dir) {
+		g_propagate_error(error, ierror);
+		return NULL;
+	}
+
+	while ((name = g_dir_read_name(dir))) {
+		g_autofree gchar *oldpath = NULL;
+		g_autofree gchar *newpath = NULL;
+
+		if (g_strcmp0(name, ".rauc-workdir") == 0)
+			continue;
+
+		oldpath = g_build_filename(contentdir, name, NULL);
+
+		/* It's not clear what symlinks in the bundle input should mean, so
+		 * reject them. If you have a use case for this, contact us. */
+		if (g_file_test(oldpath, G_FILE_TEST_IS_SYMLINK)) {
+			g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_PAYLOAD,
+					"symlinks are not supported as bundle contents (%s)", name);
+			return NULL;
+		}
+
+		/* Directories should not be needed in the bundle input, so reject
+		 * them. If you have a use case for this, contact us. */
+		if (g_file_test(oldpath, G_FILE_TEST_IS_DIR)) {
+			g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_PAYLOAD,
+					"directories are not supported as bundle contents (%s)", name);
+			return NULL;
+		}
+
+		if (!g_file_test(oldpath, G_FILE_TEST_IS_REGULAR)) {
+			g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_PAYLOAD,
+					"only regular files are supported as bundle contents (%s)", name);
+			return NULL;
+		}
+
+		newpath = g_build_filename(workdir, name, NULL);
+
+		if (link(oldpath, newpath) != 0) {
+			int err = errno;
+			g_set_error(error, G_FILE_ERROR, g_file_error_from_errno(err),
+					"failed to hard-link file '%s' to workdir: %s", name, g_strerror(err));
+			return NULL;
+		}
+	}
+
+	return g_steal_pointer(&workdir);
+}
+
 gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError **error)
 {
 	GError *ierror = NULL;
-	g_autofree gchar* manifestpath = g_build_filename(contentdir, "manifest.raucm", NULL);
+	g_autofree gchar* manifestpath = NULL;
 	g_autoptr(RaucManifest) manifest = NULL;
+	g_autofree gchar *workdir = NULL;
 	gboolean res = FALSE;
 
 	g_return_val_if_fail(bundlename != NULL, FALSE);
@@ -894,26 +983,34 @@ gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError
 		return FALSE;
 	}
 
+	workdir = prepare_workdir(contentdir, &ierror);
+	if (!workdir) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	manifestpath = g_build_filename(workdir, "manifest.raucm", NULL);
 	res = load_manifest_file(manifestpath, &manifest, &ierror);
 	if (!res) {
-		g_propagate_error(error, ierror);
+		g_propagate_prefixed_error(error, ierror,
+				"Failed to load manifest: ");
 		goto out;
 	}
 
 	g_print("Creating '%s' format bundle\n", r_manifest_bundle_format_to_str(manifest->bundle_format));
 
 	/* print warnings collected while parsing */
-	for (guint i =  0; i < manifest->warnings->len; i++) {
+	for (guint i = 0; i < manifest->warnings->len; i++) {
 		g_print("%s\n", (gchar *)g_ptr_array_index(manifest->warnings, i));
 	}
 
-	res = sync_manifest_with_contentdir(manifest, contentdir, &ierror);
+	res = sync_manifest_with_contentdir(manifest, workdir, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
 	}
 
-	res = generate_adaptive_data(manifest, contentdir, &ierror);
+	res = generate_adaptive_data(manifest, workdir, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -925,7 +1022,7 @@ gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError
 		goto out;
 	}
 
-	res = mksquashfs(bundlename, contentdir, &ierror);
+	res = mksquashfs(bundlename, workdir, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -945,6 +1042,11 @@ gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError
 		goto out;
 	}
 
+	if (workdir && !rm_tree(workdir, &ierror)) {
+		g_propagate_prefixed_error(error, ierror, "Failed to remove workdir: ");
+		goto out;
+	}
+
 	res = TRUE;
 
 out:
@@ -1126,7 +1228,8 @@ static gboolean convert_to_casync_bundle(RaucBundle *bundle, const gchar *outbun
 	/* Load manifest from content/ dir */
 	res = load_manifest_file(mfpath, &manifest, &ierror);
 	if (!res) {
-		g_propagate_error(error, ierror);
+		g_propagate_prefixed_error(error, ierror,
+				"Failed to load manifest: ");
 		goto out;
 	}
 
@@ -1252,7 +1355,7 @@ out:
 gboolean encrypt_bundle(RaucBundle *bundle, const gchar *outbundle, GError **error)
 {
 	GError *ierror = NULL;
-	GBytes *encdata = NULL;
+	g_autoptr(GBytes) encdata = NULL;
 	gboolean res = FALSE;
 
 	g_return_val_if_fail(bundle != NULL, FALSE);
@@ -1482,6 +1585,7 @@ static gboolean check_bundle_access(int bundle_fd, GError **error)
 			case JFFS2_SUPER_MAGIC:
 			case MSDOS_SUPER_MAGIC:
 			case NTFS_SB_MAGIC:
+			case NTFS3_SUPER_MAGIC:
 			case ROMFS_MAGIC:
 			case SQUASHFS_MAGIC:
 			case UDF_SUPER_MAGIC:
@@ -2219,7 +2323,8 @@ gboolean replace_signature(RaucBundle *bundle, const gchar *insig, const gchar *
 	} else {
 		res = load_manifest_from_bundle(bundle, &loaded_manifest, &ierror);
 		if (!res) {
-			g_propagate_error(error, ierror);
+			g_propagate_prefixed_error(error, ierror,
+					"Failed to load manifest: ");
 			goto out;
 		}
 		manifest = loaded_manifest;
