@@ -22,6 +22,7 @@
 #include "mount.h"
 #include "service.h"
 #include "signature.h"
+#include "slot.h"
 #include "status_file.h"
 #include "update_handler.h"
 #include "utils.h"
@@ -264,7 +265,7 @@ gboolean determine_boot_states(GError **error)
 	/* get boot state */
 	g_hash_table_iter_init(&iter, r_context()->config->slots);
 	while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &slot)) {
-		GError *ierror = NULL;
+		g_autoptr(GError) ierror = NULL;
 
 		if (!slot->bootname)
 			continue;
@@ -550,6 +551,49 @@ static gboolean verify_compatible(RaucInstallArgs *args, RaucManifest *manifest,
 	}
 }
 
+static gboolean check_version_limits(RaucInstallArgs *args, RaucManifest *manifest, GError **error)
+{
+	GError *ierror = NULL;
+
+	g_return_val_if_fail(args, FALSE);
+	g_return_val_if_fail(manifest, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	if (!r_context()->config->system_min_bundle_version) {
+		g_debug("No min-bundle-version configured in the system.conf. Version-limit check does nothing.");
+		return TRUE;
+	}
+
+	if (args->ignore_version_limit) {
+		g_message("Explicitly ignoring bundle version limit check.");
+		return TRUE;
+	}
+
+	if (!manifest->update_version) {
+		g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_VERSION_MISMATCH,
+				"Version mismatch: Expected at least '%s' but bundle manifest has no version",
+				r_context()->config->system_min_bundle_version);
+		return FALSE;
+	}
+
+	if (!r_semver_less_equal(
+			r_context()->config->system_min_bundle_version,
+			manifest->update_version,
+			&ierror)) {
+		if (ierror) {
+			g_propagate_error(error, ierror);
+			return FALSE;
+		} else {
+			g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_VERSION_MISMATCH,
+					"Version mismatch: Expected at least '%s' but bundle manifest has '%s'",
+					r_context()->config->system_min_bundle_version,
+					manifest->update_version);
+			return FALSE;
+		}
+	}
+	return TRUE;
+}
+
 static gchar **add_system_environment(gchar **envp)
 {
 	GHashTableIter iter;
@@ -777,6 +821,7 @@ static gboolean run_bundle_hook(RaucManifest *manifest, gchar* bundledir, const
 
 	g_subprocess_launcher_setenv(launcher, "RAUC_MF_COMPATIBLE", manifest->update_compatible, TRUE);
 	g_subprocess_launcher_setenv(launcher, "RAUC_MF_VERSION", manifest->update_version ?: "", TRUE);
+	g_subprocess_launcher_setenv(launcher, "RAUC_MF_BUILD", manifest->update_build ?: "", TRUE);
 	g_subprocess_launcher_setenv(launcher, "RAUC_MOUNT_PREFIX", r_context()->config->mount_prefix, TRUE);
 
 	sproc = g_subprocess_launcher_spawn(
@@ -915,14 +960,14 @@ static gboolean pre_install_checks(gchar* bundledir, GPtrArray *install_plans, G
 		}
 
 		if (!g_file_test(plan->image->filename, G_FILE_TEST_EXISTS)) {
-			g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_NOSRC,
+			g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT,
 					"Source image '%s' not found in bundle", plan->image->filename);
 			return FALSE;
 		}
 
 skip_filename_checks:
 		if (!g_file_test(plan->target_slot->device, G_FILE_TEST_EXISTS)) {
-			g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_NODST,
+			g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT,
 					"Destination device '%s' not found", plan->target_slot->device);
 			return FALSE;
 		}
@@ -979,30 +1024,14 @@ static gboolean handle_slot_install_plan(const RaucManifest *manifest, const RIm
 		return FALSE;
 	}
 
-	/* For global slot status: Clear checksum info and make status
-	 * 'pending' to prevent the slot status from looking valid later in
-	 * case we crash while installing. */
-	if (g_strcmp0(r_context()->config->statusfile_path, "per-slot") != 0) {
-		g_clear_pointer(&slot_state->status, g_free);
-		slot_state->status = g_strdup("pending");
-		g_clear_pointer(&slot_state->checksum.digest, g_free);
-		slot_state->checksum.size = 0;
-
-		if (!r_slot_status_save(plan->target_slot, &ierror)) {
-			g_propagate_prefixed_error(error, ierror, "Error while writing status file: ");
-			r_context_end_step("check_slot", FALSE);
-			return FALSE;
-		}
-	}
-
 	/* if explicitly enabled, skip update of up-to-date slots */
 	if (!plan->target_slot->install_same && g_strcmp0(plan->image->checksum.digest, slot_state->checksum.digest) == 0) {
 		install_args_update(args, "Skipping update for correct image %s", plan->image->filename);
 		g_message("Skipping update for correct image %s", plan->image->filename);
 		r_context_end_step("check_slot", TRUE);
 
-		/* Dummy step to indicate slot was skipped */
-		r_context_begin_step("skip_image", "Copying image skipped", 0);
+		/* Dummy step to indicate slot was skipped and complete required 'update_slots' substeps */
+		r_context_begin_step_weighted_formatted("skip_image", 0, 9, "Copying image skipped");
 
 		/* Update the status also for skipped slots */
 		g_message("Updating slot %s status", plan->target_slot->name);
@@ -1019,6 +1048,28 @@ static gboolean handle_slot_install_plan(const RaucManifest *manifest, const RIm
 		return TRUE;
 	}
 
+	/* For global slot status: Clear checksum info and make status
+	 * 'pending' to prevent the slot status from looking valid later in
+	 * case we crash while installing. */
+	if (g_strcmp0(r_context()->config->statusfile_path, "per-slot") != 0) {
+		g_clear_pointer(&slot_state->status, g_free);
+		slot_state->status = g_strdup("pending");
+
+		g_autofree gchar *old_digest = g_steal_pointer(&slot_state->checksum.digest);
+		slot_state->checksum.size = 0;
+
+		if (!r_slot_move_checksum_data_directory(plan->target_slot, old_digest, NULL, &ierror)) {
+			g_warning("Failed to move slot data directory, but will try to continue: %s", ierror->message);
+			g_clear_error(&ierror);
+		}
+
+		if (!r_slot_status_save(plan->target_slot, &ierror)) {
+			g_propagate_prefixed_error(error, ierror, "Error while writing status file: ");
+			r_context_end_step("check_slot", FALSE);
+			return FALSE;
+		}
+	}
+
 	g_free(slot_state->status);
 	slot_state->status = g_strdup("update");
 
@@ -1123,8 +1174,9 @@ static void log_event_installation_done(RaucInstallArgs *args, RaucManifest *man
 {
 	g_autofree gchar *formatted = NULL;
 	GLogField fields[] = {
-		{"MESSAGE", NULL, -1 },
-		{"MESSAGE_ID", NULL, -1 },
+		{"MESSAGE", NULL, -1},
+		{"MESSAGE_ID", NULL, -1},
+		{"PRIORITY", r_event_log_level_to_priority(G_LOG_LEVEL_MESSAGE), -1},
 		{"GLIB_DOMAIN", R_EVENT_LOG_DOMAIN, -1},
 		{"RAUC_EVENT_TYPE", "install", -1},
 		{"BUNDLE_HASH", "", -1},
@@ -1151,9 +1203,9 @@ static void log_event_installation_done(RaucInstallArgs *args, RaucManifest *man
 
 	fields[0].value = formatted;
 	if (manifest) {
-		fields[4].value = manifest->hash ?: "";
-		fields[5].value = manifest->update_description ?: "";
-		fields[6].value = manifest->update_version ?: "";
+		fields[5].value = manifest->hash ?: "";
+		fields[6].value = manifest->update_description ?: "";
+		fields[7].value = manifest->update_version ?: "";
 	}
 
 	g_log_structured_array(G_LOG_LEVEL_MESSAGE, fields, G_N_ELEMENTS(fields));
@@ -1414,6 +1466,12 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 		goto umount;
 	}
 
+	if (!check_version_limits(args, bundle->manifest, &ierror)) {
+		res = FALSE;
+		g_propagate_error(error, ierror);
+		goto umount;
+	}
+
 	if (bundle->manifest->handler_name) {
 		g_message("Using custom handler: %s", bundle->manifest->handler_name);
 		res = launch_and_wait_custom_handler(args, bundle->mount_point, bundle->manifest, target_group, &ierror);
