@@ -2,6 +2,7 @@
 #include <gio/gio.h>
 #include <glib.h>
 #include <glib/gstdio.h>
+#include <glib-unix.h>
 #if ENABLE_JSON
 #include <json-glib/json-glib.h>
 #include <json-glib/json-gobject.h>
@@ -29,7 +30,7 @@
 GMainLoop *r_loop = NULL;
 int r_exit_status = 0;
 
-gboolean install_ignore_compatible, install_progressbar = FALSE;
+gboolean install_ignore_compatible, install_ignore_version_limit, install_progressbar = FALSE;
 gboolean trust_environment = FALSE;
 gboolean verification_disabled = FALSE;
 gboolean no_check_time = FALSE;
@@ -210,6 +211,18 @@ static void print_progress_callback(gint percentage,
 	g_print("%3"G_GINT32_FORMAT "%% %s\n", percentage, message);
 }
 
+static gboolean on_sigint(gpointer user_data)
+{
+	RaucInstallArgs *args = user_data;
+
+	g_mutex_lock(&args->status_mutex);
+	args->status_result = 3;
+	g_mutex_unlock(&args->status_mutex);
+
+	args->cleanup(args);
+	return G_SOURCE_REMOVE;
+}
+
 static gboolean install_start(int argc, char **argv)
 {
 	GBusType bus_type = (!g_strcmp0(g_getenv("DBUS_STARTER_BUS_TYPE"), "session"))
@@ -246,6 +259,7 @@ static gboolean install_start(int argc, char **argv)
 	args->status_result = 2;
 
 	args->ignore_compatible = install_ignore_compatible;
+	args->ignore_version_limit = install_ignore_version_limit;
 	args->transaction = installation_txn;
 	if (access_args.tls_cert)
 		args->access_args.tls_cert = g_strdup(access_args.tls_cert);
@@ -262,7 +276,10 @@ static gboolean install_start(int argc, char **argv)
 	if (ENABLE_SERVICE) {
 		g_auto(GVariantDict) dict = G_VARIANT_DICT_INIT(NULL);
 
+		g_unix_signal_add(SIGINT, on_sigint, args);
+
 		g_variant_dict_insert(&dict, "ignore-compatible", "b", args->ignore_compatible);
+		g_variant_dict_insert(&dict, "ignore-version-limit", "b", args->ignore_version_limit);
 		if (args->transaction)
 			g_variant_dict_insert(&dict, "transaction-id", "s", args->transaction);
 		if (args->access_args.tls_cert)
@@ -301,7 +318,9 @@ static gboolean install_start(int argc, char **argv)
 				g_variant_dict_end(&dict), /* floating, no unref needed */
 				NULL,
 				&error)) {
-			g_printerr("Failed %s\n", error->message);
+			if (g_dbus_error_is_remote_error(error))
+				g_dbus_error_strip_remote_error(error);
+			g_printerr("Failed to contact rauc service: %s\n", error->message);
 			g_error_free(error);
 			goto out_loop;
 		}
@@ -331,6 +350,10 @@ out_loop:
 		case 2:
 			g_printerr("D-Bus error while installing `%s`\n", args->name);
 			break;
+		case 3:
+			g_print("\nCtrl+C pressed. Exiting rauc installation client...\n"
+					"Note that this will not abort the installation running in the rauc service!\n");
+			break;
 		default:
 			g_printerr("Installing `%s` failed with unknown exit code: %d\n", args->name, args->status_result);
 			break;
@@ -419,7 +442,7 @@ out:
 static gboolean write_slot_start(int argc, char **argv)
 {
 	GError *ierror = NULL;
-	g_autoptr(RaucImage) image = g_new0(RaucImage, 1);
+	g_autoptr(RaucImage) image = r_new_image();
 	RaucSlot *slot = NULL;
 	g_autoptr(GFileInfo) info = NULL;
 	g_autoptr(GInputStream) instream = NULL;
@@ -564,7 +587,7 @@ G_GNUC_UNUSED
 static gboolean replace_signature_start(int argc, char **argv)
 {
 	CheckBundleParams check_bundle_params = CHECK_BUNDLE_DEFAULT;
-	RaucBundle *bundle = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
 	GError *ierror = NULL;
 	g_debug("replace signature start");
 
@@ -623,7 +646,7 @@ G_GNUC_UNUSED
 static gboolean extract_signature_start(int argc, char **argv)
 {
 	CheckBundleParams check_bundle_params = CHECK_BUNDLE_DEFAULT;
-	RaucBundle *bundle = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
 	GError *ierror = NULL;
 	g_debug("extract signature start");
 
@@ -672,7 +695,7 @@ out:
 static gboolean extract_start(int argc, char **argv)
 {
 	CheckBundleParams check_bundle_params = CHECK_BUNDLE_DEFAULT;
-	RaucBundle *bundle = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
 	GError *ierror = NULL;
 	g_debug("extract start");
 
@@ -722,7 +745,7 @@ G_GNUC_UNUSED
 static gboolean convert_start(int argc, char **argv)
 {
 	CheckBundleParams check_bundle_params = CHECK_BUNDLE_DEFAULT;
-	RaucBundle *bundle = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
 	GError *ierror = NULL;
 	g_debug("convert start");
 
@@ -782,7 +805,7 @@ out:
 G_GNUC_UNUSED
 static gboolean encrypt_start(int argc, char **argv)
 {
-	RaucBundle *bundle = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
 	GError *ierror = NULL;
 	g_debug("encrypt start");
 
@@ -844,43 +867,29 @@ out:
 #define KWHT  "\x1B[37m"
 #define KBLD  "\x1B[1m"
 
-/* Takes a shell variable and its desired argument as input and appends it to
- * the provided text with taking care of correct shell quoting */
-static void formatter_shell_append(GString* text, const gchar* varname, const gchar* argument)
-{
-	g_autofree gchar* quoted = g_shell_quote(argument ?: "");
-	g_string_append_printf(text, "%s=%s\n", varname, quoted);
-}
-/* Same as above, expect that it has a cnt argument to add per-slot-number
- * strings */
-static void formatter_shell_append_n(GString* text, const gchar* varname, gint cnt, const gchar* argument)
-{
-	g_autofree gchar* quoted = g_shell_quote(argument ?: "");
-	g_string_append_printf(text, "%s_%d=%s\n", varname, cnt, quoted);
-}
-
 static gchar *info_formatter_shell(RaucManifest *manifest)
 {
-	GString *text = g_string_new(NULL);
+	g_autoptr(GPtrArray) entries = g_ptr_array_new_with_free_func(g_free);
 	GPtrArray *hooks = NULL;
 	gchar *temp_string = NULL;
 	gint cnt;
 
-	formatter_shell_append(text, "RAUC_MF_COMPATIBLE", manifest->update_compatible);
-	formatter_shell_append(text, "RAUC_MF_VERSION", manifest->update_version);
-	formatter_shell_append(text, "RAUC_MF_DESCRIPTION", manifest->update_description);
-	formatter_shell_append(text, "RAUC_MF_BUILD", manifest->update_build);
-	formatter_shell_append(text, "RAUC_MF_HASH", manifest->hash);
-	g_string_append_printf(text, "RAUC_MF_IMAGES=%d\n", g_list_length(manifest->images));
+	r_ptr_array_add_printf(entries, "RAUC_MF_COMPATIBLE=%s", manifest->update_compatible ?: "");
+	r_ptr_array_add_printf(entries, "RAUC_MF_VERSION=%s", manifest->update_version ?: "");
+	r_ptr_array_add_printf(entries, "RAUC_MF_DESCRIPTION=%s", manifest->update_description ?: "");
+	r_ptr_array_add_printf(entries, "RAUC_MF_BUILD=%s", manifest->update_build ?: "");
+	r_ptr_array_add_printf(entries, "RAUC_MF_HASH=%s", manifest->hash ?: "");
+	r_ptr_array_add_printf(entries, "RAUC_MF_FORMAT=%s", r_manifest_bundle_format_to_str(manifest->bundle_format));
+	r_ptr_array_add_printf(entries, "RAUC_MF_IMAGES=%d", g_list_length(manifest->images));
 
 	hooks = g_ptr_array_new();
 	if (manifest->hooks.install_check == TRUE) {
-		g_ptr_array_add(hooks, g_strdup("install-check"));
+		g_ptr_array_add(hooks, (gchar*) "install-check");
 	}
 	g_ptr_array_add(hooks, NULL);
 
 	temp_string = g_strjoinv(" ", (gchar**) hooks->pdata);
-	formatter_shell_append(text, "RAUC_MF_HOOKS", temp_string);
+	r_ptr_array_add_printf(entries, "RAUC_MF_HOOKS=%s", temp_string);
 	g_free(temp_string);
 
 	g_ptr_array_unref(hooks);
@@ -902,13 +911,11 @@ static gchar *info_formatter_shell(RaucManifest *manifest)
 			g_hash_table_iter_init(&kvs_iter, kvs);
 			while (g_hash_table_iter_next(&kvs_iter, (gpointer*)&key, (gpointer*)&value)) {
 				g_autofree gchar *env_key = r_prepare_env_key(key, NULL);
-				g_autofree gchar *var = NULL;
 
 				if (!env_key)
 					continue;
 
-				var = g_strdup_printf("RAUC_META_%s_%s", env_group, env_key);
-				formatter_shell_append(text, var, value);
+				r_ptr_array_add_printf(entries, "RAUC_META_%s_%s=%s", env_group, env_key, value);
 			}
 		}
 	}
@@ -916,40 +923,40 @@ static gchar *info_formatter_shell(RaucManifest *manifest)
 	cnt = 0;
 	for (GList *l = manifest->images; l != NULL; l = l->next) {
 		RaucImage *img = l->data;
-		formatter_shell_append_n(text, "RAUC_IMAGE_NAME", cnt, img->filename);
-		formatter_shell_append_n(text, "RAUC_IMAGE_CLASS", cnt, img->slotclass);
-		formatter_shell_append_n(text, "RAUC_IMAGE_VARIANT", cnt, img->variant);
-		formatter_shell_append_n(text, "RAUC_IMAGE_DIGEST", cnt, img->checksum.digest);
-		g_string_append_printf(text, "RAUC_IMAGE_SIZE_%d=%"G_GOFFSET_FORMAT "\n", cnt, img->checksum.size);
+		r_ptr_array_add_printf(entries, "RAUC_IMAGE_NAME_%d=%s", cnt, img->filename ?: "");
+		r_ptr_array_add_printf(entries, "RAUC_IMAGE_CLASS_%d=%s", cnt, img->slotclass ?: "");
+		r_ptr_array_add_printf(entries, "RAUC_IMAGE_VARIANT_%d=%s", cnt, img->variant ?: "");
+		r_ptr_array_add_printf(entries, "RAUC_IMAGE_DIGEST_%d=%s", cnt, img->checksum.digest ?: "");
+		r_ptr_array_add_printf(entries, "RAUC_IMAGE_SIZE_%d=%"G_GOFFSET_FORMAT, cnt, img->checksum.size);
 
 		hooks = g_ptr_array_new();
 		if (img->hooks.pre_install == TRUE) {
-			g_ptr_array_add(hooks, g_strdup("pre-install"));
+			g_ptr_array_add(hooks, (gchar*) "pre-install");
 		}
 		if (img->hooks.install == TRUE) {
-			g_ptr_array_add(hooks, g_strdup("install"));
+			g_ptr_array_add(hooks, (gchar*) "install");
 		}
 		if (img->hooks.post_install == TRUE) {
-			g_ptr_array_add(hooks, g_strdup("post-install"));
+			g_ptr_array_add(hooks, (gchar*) "post-install");
 		}
 		g_ptr_array_add(hooks, NULL);
 
 		temp_string = g_strjoinv(" ", (gchar**) hooks->pdata);
-		formatter_shell_append_n(text, "RAUC_IMAGE_HOOKS", cnt, temp_string);
+		r_ptr_array_add_printf(entries, "RAUC_IMAGE_HOOKS_%d=%s", cnt, temp_string);
 		g_free(temp_string);
 
 		g_ptr_array_unref(hooks);
 
 		if (img->adaptive) {
 			temp_string = g_strjoinv(" ", (gchar**) img->adaptive);
-			formatter_shell_append_n(text, "RAUC_IMAGE_ADAPTIVE", cnt, temp_string);
+			r_ptr_array_add_printf(entries, "RAUC_IMAGE_ADAPTIVE_%d=%s", cnt, temp_string);
 			g_free(temp_string);
 		}
 
 		cnt++;
 	}
 
-	return g_string_free(text, FALSE);
+	return r_ptr_array_env_to_shell(entries);
 }
 
 static gchar *info_formatter_readable(RaucManifest *manifest)
@@ -967,7 +974,7 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 
 	hooks = g_ptr_array_new();
 	if (manifest->hooks.install_check == TRUE) {
-		g_ptr_array_add(hooks, g_strdup("install-check"));
+		g_ptr_array_add(hooks, (gchar*) "install-check");
 	}
 	g_ptr_array_add(hooks, NULL);
 
@@ -1036,13 +1043,13 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 
 		hooks = g_ptr_array_new();
 		if (img->hooks.pre_install == TRUE) {
-			g_ptr_array_add(hooks, g_strdup("pre-install"));
+			g_ptr_array_add(hooks, (gchar*) "pre-install");
 		}
 		if (img->hooks.install == TRUE) {
-			g_ptr_array_add(hooks, g_strdup("install"));
+			g_ptr_array_add(hooks, (gchar*) "install");
 		}
 		if (img->hooks.post_install == TRUE) {
-			g_ptr_array_add(hooks, g_strdup("post-install"));
+			g_ptr_array_add(hooks, (gchar*) "post-install");
 		}
 		g_ptr_array_add(hooks, NULL);
 
@@ -1103,6 +1110,9 @@ static gchar* info_formatter_json_base(RaucManifest *manifest, gboolean pretty)
 	json_builder_set_member_name(builder, "build");
 	json_builder_add_string_value(builder, manifest->update_build);
 
+	json_builder_set_member_name(builder, "format");
+	json_builder_add_string_value(builder, r_manifest_bundle_format_to_str(manifest->bundle_format));
+
 	json_builder_set_member_name(builder, "hooks");
 	json_builder_begin_array(builder);
 	if (manifest->hooks.install_check == TRUE) {
@@ -1309,7 +1319,7 @@ static void free_status_print(RaucStatusPrint *status)
 	g_free(status->variant);
 	g_free(status->bootslot);
 	if (status->slots)
-		g_hash_table_destroy(status->slots);
+		g_hash_table_unref(status->slots);
 
 	g_free(status);
 }
@@ -1438,7 +1448,7 @@ static gchar* r_status_formatter_shell(RaucStatusPrint *status)
 {
 	GHashTableIter iter;
 	gint slotcnt = 0;
-	GString *text = g_string_new(NULL);
+	g_autoptr(GPtrArray) entries = g_ptr_array_new_with_free_func(g_free);
 	GPtrArray *slotnames, *slotnumbers = NULL;
 	gchar* slotstring = NULL;
 	RaucSlot *slot = NULL;
@@ -1446,10 +1456,10 @@ static gchar* r_status_formatter_shell(RaucStatusPrint *status)
 
 	g_return_val_if_fail(status, NULL);
 
-	formatter_shell_append(text, "RAUC_SYSTEM_COMPATIBLE", status->compatible);
-	formatter_shell_append(text, "RAUC_SYSTEM_VARIANT", status->variant);
-	formatter_shell_append(text, "RAUC_SYSTEM_BOOTED_BOOTNAME", status->bootslot);
-	formatter_shell_append(text, "RAUC_BOOT_PRIMARY", status->primary ? status->primary->name : NULL);
+	r_ptr_array_add_printf(entries, "RAUC_SYSTEM_COMPATIBLE=%s", status->compatible);
+	r_ptr_array_add_printf(entries, "RAUC_SYSTEM_VARIANT=%s", status->variant);
+	r_ptr_array_add_printf(entries, "RAUC_SYSTEM_BOOTED_BOOTNAME=%s", status->bootslot);
+	r_ptr_array_add_printf(entries, "RAUC_BOOT_PRIMARY=%s", status->primary ? status->primary->name : NULL);
 
 	slotnames = g_ptr_array_new();
 	slotnumbers = g_ptr_array_new_with_free_func(g_free);
@@ -1462,10 +1472,10 @@ static gchar* r_status_formatter_shell(RaucStatusPrint *status)
 	g_ptr_array_add(slotnumbers, NULL);
 
 	slotstring = g_strjoinv(" ", (gchar**) slotnames->pdata);
-	formatter_shell_append(text, "RAUC_SYSTEM_SLOTS", slotstring);
+	r_ptr_array_add_printf(entries, "RAUC_SYSTEM_SLOTS=%s", slotstring);
 	g_free(slotstring);
 	slotstring = g_strjoinv(" ", (gchar**) slotnumbers->pdata);
-	formatter_shell_append(text, "RAUC_SLOTS", slotstring);
+	r_ptr_array_add_printf(entries, "RAUC_SLOTS=%s", slotstring);
 	g_free(slotstring);
 
 	g_ptr_array_unref(slotnumbers);
@@ -1478,42 +1488,34 @@ static gchar* r_status_formatter_shell(RaucStatusPrint *status)
 
 		slotcnt++;
 
-		formatter_shell_append_n(text, "RAUC_SLOT_STATE", slotcnt, r_slot_slotstate_to_str(slot->state));
-		formatter_shell_append_n(text, "RAUC_SLOT_CLASS", slotcnt, slot->sclass);
-		formatter_shell_append_n(text, "RAUC_SLOT_DEVICE", slotcnt, slot->device);
-		formatter_shell_append_n(text, "RAUC_SLOT_TYPE", slotcnt, slot->type);
-		formatter_shell_append_n(text, "RAUC_SLOT_BOOTNAME", slotcnt, slot->bootname);
-		formatter_shell_append_n(text, "RAUC_SLOT_PARENT", slotcnt, slot->parent ? slot->parent->name : NULL);
-		formatter_shell_append_n(text, "RAUC_SLOT_MOUNTPOINT", slotcnt, slot->mount_point);
+		r_ptr_array_add_printf(entries, "RAUC_SLOT_STATE_%d=%s", slotcnt, r_slot_slotstate_to_str(slot->state));
+		r_ptr_array_add_printf(entries, "RAUC_SLOT_CLASS_%d=%s", slotcnt, slot->sclass);
+		r_ptr_array_add_printf(entries, "RAUC_SLOT_DEVICE_%d=%s", slotcnt, slot->device);
+		r_ptr_array_add_printf(entries, "RAUC_SLOT_TYPE_%d=%s", slotcnt, slot->type);
+		r_ptr_array_add_printf(entries, "RAUC_SLOT_BOOTNAME_%d=%s", slotcnt, slot->bootname ?: "");
+		r_ptr_array_add_printf(entries, "RAUC_SLOT_PARENT_%d=%s", slotcnt, slot->parent ? slot->parent->name : "");
+		r_ptr_array_add_printf(entries, "RAUC_SLOT_MOUNTPOINT_%d=%s", slotcnt, slot->mount_point ?: "");
 		if (slot->bootname)
-			formatter_shell_append_n(text, "RAUC_SLOT_BOOT_STATUS", slotcnt, slot->boot_good ? "good" : "bad");
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_BOOT_STATUS_%d=%s", slotcnt, slot->boot_good ? "good" : "bad");
 		else
-			formatter_shell_append_n(text, "RAUC_SLOT_BOOT_STATUS", slotcnt, NULL);
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_BOOT_STATUS_%d=", slotcnt);
 		if (status_detailed && slot_state) {
-			gchar *str;
-
-			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_BUNDLE_COMPATIBLE", slotcnt, slot_state->bundle_compatible);
-			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_BUNDLE_VERSION", slotcnt, slot_state->bundle_version);
-			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_BUNDLE_DESCRIPTION", slotcnt, slot_state->bundle_description);
-			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_BUNDLE_BUILD", slotcnt, slot_state->bundle_build);
-			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_BUNDLE_HASH", slotcnt, slot_state->bundle_hash);
-			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_CHECKSUM_SHA256", slotcnt, slot_state->checksum.digest);
-			str = g_strdup_printf("%"G_GOFFSET_FORMAT, slot_state->checksum.size);
-			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_CHECKSUM_SIZE", slotcnt, str);
-			g_free(str);
-			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_INSTALLED_TIMESTAMP", slotcnt, slot_state->installed_timestamp);
-			str = g_strdup_printf("%u", slot_state->installed_count);
-			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_INSTALLED_COUNT", slotcnt, str);
-			g_free(str);
-			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_ACTIVATED_TIMESTAMP", slotcnt, slot_state->activated_timestamp);
-			str = g_strdup_printf("%u", slot_state->activated_count);
-			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_ACTIVATED_COUNT", slotcnt, str);
-			g_free(str);
-			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_STATUS", slotcnt, slot_state->status);
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_STATUS_BUNDLE_COMPATIBLE_%d=%s", slotcnt, slot_state->bundle_compatible ?: "");
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_STATUS_BUNDLE_VERSION_%d=%s", slotcnt, slot_state->bundle_version ?: "");
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_STATUS_BUNDLE_DESCRIPTION_%d=%s", slotcnt, slot_state->bundle_description ?: "");
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_STATUS_BUNDLE_BUILD_%d=%s", slotcnt, slot_state->bundle_build ?: "");
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_STATUS_BUNDLE_HASH_%d=%s", slotcnt, slot_state->bundle_hash ?: "");
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_STATUS_CHECKSUM_SHA256_%d=%s", slotcnt, slot_state->checksum.digest ?: "");
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_STATUS_CHECKSUM_SIZE_%d=%"G_GOFFSET_FORMAT, slotcnt, slot_state->checksum.size);
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_STATUS_INSTALLED_TIMESTAMP_%d=%s", slotcnt, slot_state->installed_timestamp ?: "");
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_STATUS_INSTALLED_COUNT_%d=%u", slotcnt, slot_state->installed_count);
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_STATUS_ACTIVATED_TIMESTAMP_%d=%s", slotcnt, slot_state->activated_timestamp ?: "");
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_STATUS_ACTIVATED_COUNT_%d=%u", slotcnt, slot_state->activated_count);
+			r_ptr_array_add_printf(entries, "RAUC_SLOT_STATUS_STATUS_%d=%s", slotcnt, slot_state->status ?: "");
 		}
 	}
 
-	return g_string_free(text, FALSE);
+	return r_ptr_array_env_to_shell(entries);
 }
 
 static gchar* r_status_formatter_json(RaucStatusPrint *status, gboolean pretty)
@@ -1923,10 +1925,10 @@ static gboolean status_start(int argc, char **argv)
 			g_clear_error(&ierror);
 		}
 
-		status_print->compatible = r_context()->config->system_compatible;
-		status_print->variant = r_context()->config->system_variant;
-		status_print->bootslot = r_context()->bootslot;
-		status_print->slots = r_context()->config->slots;
+		status_print->compatible = g_strdup(r_context()->config->system_compatible);
+		status_print->variant = g_strdup(r_context()->config->system_variant);
+		status_print->bootslot = g_strdup(r_context()->bootslot);
+		status_print->slots = g_hash_table_ref(r_context()->config->slots);
 	} else {
 		if (!retrieve_status_via_dbus(&status_print, &ierror)) {
 			g_printerr("Error retrieving slot status via D-Bus: %s\n",
@@ -2010,8 +2012,11 @@ static void r_event_log_booted(const RaucSlot *booted_slot)
 	GLogField fields[] = {
 		{"MESSAGE", NULL, -1 },
 		{"MESSAGE_ID", MESSAGE_ID_BOOTED, -1 },
+		{"PRIORITY", r_event_log_level_to_priority(G_LOG_LEVEL_MESSAGE), -1},
 		{"GLIB_DOMAIN", R_EVENT_LOG_DOMAIN, -1},
 		{"RAUC_EVENT_TYPE", "boot", -1},
+		{"SLOT_NAME", NULL, -1},
+		{"SLOT_BOOTNAME", NULL, -1},
 		{"BOOT_ID", NULL, -1},
 		{"BUNDLE_HASH", NULL, -1},
 	};
@@ -2020,13 +2025,15 @@ static void r_event_log_booted(const RaucSlot *booted_slot)
 
 	message = g_strdup_printf("Booted into %s (%s)", booted_slot->name, booted_slot->bootname);
 	fields[0].value = message;
-	fields[4].value = r_context()->boot_id;
+	fields[5].value = booted_slot->name;
+	fields[6].value = booted_slot->bootname;
+	fields[7].value = r_context()->boot_id;
 	if (booted_slot->status && booted_slot->status->bundle_hash) {
-		fields[5].value =  booted_slot->status->bundle_hash;
+		fields[8].value = booted_slot->status->bundle_hash;
 	} else {
-		fields[5].value =  "unknown";
+		fields[8].value = "unknown";
 	}
-	g_log_structured_array(G_LOG_LEVEL_INFO, fields, G_N_ELEMENTS(fields));
+	g_log_structured_array(G_LOG_LEVEL_MESSAGE, fields, G_N_ELEMENTS(fields));
 }
 
 static void r_event_log_booted_external(void)
@@ -2035,6 +2042,7 @@ static void r_event_log_booted_external(void)
 	GLogField fields[] = {
 		{"MESSAGE", NULL, -1 },
 		{"MESSAGE_ID", MESSAGE_ID_BOOTED_EXTERNAL, -1 },
+		{"PRIORITY", r_event_log_level_to_priority(G_LOG_LEVEL_MESSAGE), -1},
 		{"GLIB_DOMAIN", R_EVENT_LOG_DOMAIN, -1},
 		{"RAUC_EVENT_TYPE", "boot", -1},
 		{"BOOT_ID", NULL, -1},
@@ -2042,8 +2050,8 @@ static void r_event_log_booted_external(void)
 
 	message = g_strdup_printf("Booted from external source");
 	fields[0].value = message;
-	fields[4].value = r_context()->boot_id;
-	g_log_structured_array(G_LOG_LEVEL_INFO, fields, G_N_ELEMENTS(fields));
+	fields[5].value = r_context()->boot_id;
+	g_log_structured_array(G_LOG_LEVEL_MESSAGE, fields, G_N_ELEMENTS(fields));
 }
 
 G_GNUC_UNUSED
@@ -2091,7 +2099,6 @@ static gboolean service_start(int argc, char **argv)
 	if (r_context()->system_status) {
 		/* Boot ID-based system reboot vs service restart detection */
 		if (g_strcmp0(r_context()->system_status->boot_id, r_context()->boot_id) == 0) {
-			g_message("Restarted RAUC service");
 			r_event_log_message(R_EVENT_LOG_TYPE_SERVICE, "Service restarted");
 		} else {
 			if (g_strcmp0(r_context()->bootslot, "_external_") == 0) {
@@ -2114,7 +2121,6 @@ static gboolean service_start(int argc, char **argv)
 			}
 		}
 	} else {
-		g_message("Started RAUC service");
 		r_event_log_message(R_EVENT_LOG_TYPE_SERVICE, "Service started");
 	}
 
@@ -2213,6 +2219,7 @@ typedef struct {
 
 static GOptionEntry entries_install[] = {
 	{"ignore-compatible", '\0', 0, G_OPTION_ARG_NONE, &install_ignore_compatible, "disable compatible check", NULL},
+	{"ignore-version-limit", '\0', 0, G_OPTION_ARG_NONE, &install_ignore_version_limit, "disable version check", NULL},
 	{"transaction-id", '\0', 0, G_OPTION_ARG_STRING, &installation_txn, "custom transaction id", "UUID"},
 #if ENABLE_SERVICE == 1
 	{"progress", '\0', 0, G_OPTION_ARG_NONE, &install_progressbar, "show progress bar", NULL},
@@ -2331,15 +2338,15 @@ static void create_option_groups(void)
 		g_option_group_add_entries(install_group, entries_bundle_access);
 
 	if (ENABLE_CREATE) {
-		bundle_group  = g_option_group_new("bundle", "Bundle options:", "help dummy", NULL, NULL);
+		bundle_group = g_option_group_new("bundle", "Bundle options:", "help dummy", NULL, NULL);
 		g_option_group_add_entries(bundle_group, entries_bundle);
 		g_option_group_add_entries(bundle_group, entries_signing);
 
-		resign_group  = g_option_group_new("resign", "Resign options:", "help dummy", NULL, NULL);
+		resign_group = g_option_group_new("resign", "Resign options:", "help dummy", NULL, NULL);
 		g_option_group_add_entries(resign_group, entries_resign);
 		g_option_group_add_entries(resign_group, entries_signing);
 
-		replace_group  = g_option_group_new("replace-signature", "Replace signature options:", "help dummy", NULL, NULL);
+		replace_group = g_option_group_new("replace-signature", "Replace signature options:", "help dummy", NULL, NULL);
 		g_option_group_add_entries(replace_group, entries_replace);
 
 		convert_group = g_option_group_new("convert", "Convert options:", "help dummy", NULL, NULL);
@@ -2356,15 +2363,15 @@ static void create_option_groups(void)
 	extract_group = g_option_group_new("extract", "Extract options:", "help dummy", NULL, NULL);
 	g_option_group_add_entries(extract_group, entries_extract);
 
-	info_group    = g_option_group_new("info", "Info options:", "help dummy", NULL, NULL);
+	info_group = g_option_group_new("info", "Info options:", "help dummy", NULL, NULL);
 	g_option_group_add_entries(info_group, entries_info);
 	if (ENABLE_STREAMING)
 		g_option_group_add_entries(info_group, entries_bundle_access);
 
-	status_group  = g_option_group_new("status", "Status options:", "help dummy", NULL, NULL);
+	status_group = g_option_group_new("status", "Status options:", "help dummy", NULL, NULL);
 	g_option_group_add_entries(status_group, entries_status);
 
-	service_group  = g_option_group_new("service", "Service options:", "help dummy", NULL, NULL);
+	service_group = g_option_group_new("service", "Service options:", "help dummy", NULL, NULL);
 	g_option_group_add_entries(service_group, entries_service);
 }
 
@@ -2498,6 +2505,15 @@ static void cmdline_handler(int argc, char **argv)
 		}
 		domains = g_getenv("G_MESSAGES_DEBUG");
 		g_message("Debug log domains: '%s'", domains);
+		g_debug(PACKAGE_VERSION
+				" create=" G_STRINGIFY(ENABLE_CREATE)
+				" emmc-boot=" G_STRINGIFY(ENABLE_EMMC_BOOT_SUPPORT)
+				" gpt=" G_STRINGIFY(ENABLE_GPT)
+				" json=" G_STRINGIFY(ENABLE_JSON)
+				" network=" G_STRINGIFY(ENABLE_NETWORK)
+				" service=" G_STRINGIFY(ENABLE_SERVICE)
+				" streaming=" G_STRINGIFY(ENABLE_STREAMING)
+				);
 	}
 
 	/* get first parameter without dashes */
@@ -2572,7 +2588,7 @@ static void cmdline_handler(int argc, char **argv)
 				r_context_conf()->configmode = R_CONTEXT_CONFIG_MODE_NONE;
 		}
 		if (confpath)
-			r_context_conf()->configpath = confpath;
+			r_context_conf()->configpath = g_steal_pointer(&confpath);
 		if (certpath)
 			r_context_conf()->certpath = certpath;
 		if (keypath) {
@@ -2584,7 +2600,7 @@ static void cmdline_handler(int argc, char **argv)
 				r_context_conf()->keypath = keypath;
 		}
 		if (keyring)
-			r_context_conf()->keyringpath = keyring;
+			r_context_conf()->keyringpath = g_steal_pointer(&keyring);
 		if (signing_keyring)
 			r_context_conf()->signing_keyringpath = signing_keyring;
 		if (mksquashfs_args)
@@ -2596,7 +2612,7 @@ static void cmdline_handler(int argc, char **argv)
 		if (intermediate)
 			r_context_conf()->intermediatepaths = intermediate;
 		if (mount)
-			r_context_conf()->mountprefix = mount;
+			r_context_conf()->mountprefix = g_steal_pointer(&mount);
 		if (bootslot)
 			r_context_conf()->bootslot = bootslot;
 		if (handler_args)
@@ -2680,5 +2696,6 @@ int main(int argc, char **argv)
 	create_option_groups();
 	cmdline_handler(argc, argv);
 
+	r_context_clean();
 	return r_exit_status;
 }
