@@ -17,7 +17,7 @@ GQuark r_manifest_error_quark(void)
 
 static gboolean parse_image(GKeyFile *key_file, const gchar *group, RaucImage **image, GError **error)
 {
-	g_autoptr(RaucImage) iimage = g_new0(RaucImage, 1);
+	g_autoptr(RaucImage) iimage = r_new_image();
 	g_auto(GStrv) groupsplit = NULL;
 	gchar *value;
 	g_auto(GStrv) hooks = NULL;
@@ -49,6 +49,7 @@ static gboolean parse_image(GKeyFile *key_file, const gchar *group, RaucImage **
 	iimage->checksum.size = g_key_file_get_uint64(key_file,
 			group, "size", &ierror);
 	if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+		/* restore size to the default of -1 */
 		iimage->checksum.size = -1;
 		g_clear_error(&ierror);
 	} else if (ierror) {
@@ -393,41 +394,6 @@ static gboolean check_manifest_common(const RaucManifest *mf, GError **error)
 		}
 	}
 
-	for (GList *l = mf->images; l != NULL; l = l->next) {
-		RaucImage *image = l->data;
-
-		g_assert(image);
-
-		/* Having no 'filename' set is valid for 'install' hook only.
-		 * This is already ensured during manifest parsing, thus simply
-		 * skip further checks here */
-		if (!image->filename)
-			continue;
-
-		if (image->checksum.type != G_CHECKSUM_SHA256) {
-			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unsupported checksum algorithm for image %s", image->filename);
-			goto out;
-		}
-		if (!image->checksum.digest) {
-			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Missing digest for image %s", image->filename);
-			goto out;
-		}
-		if (image->checksum.size < 0) {
-			/* RAUC versions before v1.5 allowed zero-size images but did not handle this explicitly.
-			 * Thus, bundles created did have a valid 'filename=' manifest entry
-			 * but the 'size=' entry was considered as empty and not set at all.
-			 * Retain support for this case, at least for the 'install' per-slot hook use-case
-			 * where an image file can be optional. */
-			if (image->hooks.install) {
-				g_message("Missing size parameter for image '%s'", image->filename);
-				image->checksum.size = 0;
-			} else {
-				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Missing size for image %s", image->filename);
-				goto out;
-			}
-		}
-	}
-
 	/* Check for hook file set if hooks are enabled */
 
 	if (mf->hooks.install_check == TRUE)
@@ -459,6 +425,46 @@ out:
 	return res;
 }
 
+static gboolean check_manifest_bundled(const RaucManifest *mf, GError **error)
+{
+	for (GList *l = mf->images; l != NULL; l = l->next) {
+		RaucImage *image = l->data;
+
+		g_assert(image);
+
+		/* Having no 'filename' set is valid for 'install' hook only.
+		 * This is already ensured during manifest parsing, thus simply
+		 * skip further checks here */
+		if (!image->filename)
+			continue;
+
+		if (image->checksum.type != G_CHECKSUM_SHA256) {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unsupported checksum algorithm for image %s", image->filename);
+			return FALSE;
+		}
+		if (!image->checksum.digest) {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Missing digest for image %s", image->filename);
+			return FALSE;
+		}
+		if (image->checksum.size < 0) {
+			/* RAUC versions before v1.5 allowed zero-size images but did not handle this explicitly.
+			 * Thus, bundles created did have a valid 'filename=' manifest entry
+			 * but the 'size=' entry was considered as empty and not set at all.
+			 * Retain support for this case, at least for the 'install' per-slot hook use-case
+			 * where an image file can be optional. */
+			if (image->hooks.install) {
+				g_message("Missing size parameter for image '%s'", image->filename);
+				image->checksum.size = 0;
+			} else {
+				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Missing size for image %s", image->filename);
+				return FALSE;
+			}
+		}
+	}
+
+	return TRUE;
+}
+
 gboolean check_manifest_internal(const RaucManifest *mf, GError **error)
 {
 	GError *ierror = NULL;
@@ -474,35 +480,36 @@ gboolean check_manifest_internal(const RaucManifest *mf, GError **error)
 	switch (mf->bundle_format) {
 		case R_MANIFEST_FORMAT_PLAIN:
 			break; /* no additional data needed */
-		case R_MANIFEST_FORMAT_CRYPT: {
-			if (mf->bundle_crypt_key) {
-				g_set_error_literal(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unexpected key for crypt bundle in internal manifest");
-				goto out;
-			}
-		};
-		/* Fallthrough */
-		case R_MANIFEST_FORMAT_VERITY: {
-			if (mf->bundle_verity_hash) {
-				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unexpected hash for %s bundle in internal manifest", r_manifest_bundle_format_to_str(mf->bundle_format));
-				goto out;
-			}
-			if (mf->bundle_verity_salt) {
-				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unexpected hash for %s bundle in internal manifest", r_manifest_bundle_format_to_str(mf->bundle_format));
-				goto out;
-			}
-			if (mf->bundle_verity_size) {
-				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unexpected hash for %s bundle in internal manifest", r_manifest_bundle_format_to_str(mf->bundle_format));
-				goto out;
-			}
-
-			break;
-		};
+		case R_MANIFEST_FORMAT_CRYPT:
+		case R_MANIFEST_FORMAT_VERITY:
 		default: {
-			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unsupported bundle format");
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Bundle format '%s' not allowed for internal manifest", r_manifest_bundle_format_to_str(mf->bundle_format));
 			goto out;
 		}
 	}
 
+	if (!check_manifest_bundled(mf, &ierror)) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	if (mf->bundle_crypt_key) {
+		g_set_error_literal(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unexpected key for crypt bundle in internal manifest");
+		goto out;
+	}
+	if (mf->bundle_verity_hash) {
+		g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unexpected verity hash for %s bundle in internal manifest", r_manifest_bundle_format_to_str(mf->bundle_format));
+		goto out;
+	}
+	if (mf->bundle_verity_salt) {
+		g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unexpected verity salt for %s bundle in internal manifest", r_manifest_bundle_format_to_str(mf->bundle_format));
+		goto out;
+	}
+	if (mf->bundle_verity_size) {
+		g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unexpected verity size for %s bundle in internal manifest", r_manifest_bundle_format_to_str(mf->bundle_format));
+		goto out;
+	}
+
 	res = TRUE;
 out:
 	r_context_end_step("check_manifest", res);
@@ -592,6 +599,11 @@ gboolean check_manifest_external(const RaucManifest *mf, GError **error)
 		}
 	}
 
+	if (!check_manifest_bundled(mf, &ierror)) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
 	res = TRUE;
 out:
 	r_context_end_step("check_manifest", res);
@@ -881,6 +893,15 @@ GVariant* r_manifest_to_dict(const RaucManifest *manifest)
 	return g_variant_dict_end(&root_dict);
 }
 
+RaucImage *r_new_image(void)
+{
+	RaucImage *image = g_new0(RaucImage, 1);
+
+	image->checksum.size = -1;
+
+	return image;
+}
+
 void r_free_image(gpointer data)
 {
 	RaucImage *image = (RaucImage*) data;
