@@ -87,6 +87,7 @@ void r_nbd_free_server(RaucNBDServer *nbd_srv)
 	g_free(nbd_srv->tls_ca);
 	g_strfreev(nbd_srv->headers);
 	g_strfreev(nbd_srv->info_headers);
+	g_free(nbd_srv->effective_url);
 	g_free(nbd_srv);
 }
 
@@ -231,7 +232,7 @@ gboolean r_nbd_remove_device(RaucNBDDevice *nbd_dev, GError **error)
 	if (!nbd_dev->index_valid)
 		return TRUE;
 
-	g_message("r_nbd_remove_device");
+	g_message("Removing nbd device %s", nbd_dev->dev);
 
 	nl = netlink_connect(&driver_id, &ierror);
 	if (!nl) {
@@ -286,13 +287,12 @@ struct RaucNBDContext {
 	gchar *tls_key; /* local file or PKCS#11 URI */
 	gchar *tls_ca; /* local file */
 	gboolean tls_no_verify;
-	GStrv headers; /* array of strings such as 'Foo: bar' */
+	struct curl_slist *headers_slist;
+	struct curl_slist *initial_headers_slist;
 
 	/* runtime state */
 	CURLM *multi;
 	gboolean done;
-	struct curl_slist *headers_slist;
-	struct curl_slist *initial_headers_slist;
 
 	/* statistics */
 	RaucStats *dl_size, *dl_speed, *namelookup, *connect, *starttransfer, *total;
@@ -384,18 +384,18 @@ static size_t header_cb(char *buffer, size_t size, size_t nitems, void *userdata
 
 		xfer->content_size = range_size;
 
-		g_message("total size %"G_GUINT64_FORMAT, range_size);
+		g_message("nbd server received total size %"G_GUINT64_FORMAT, range_size);
 	} else if (g_str_equal(h_pair[0], "date")) {
 		time_t date = curl_getdate(h_pair[1], NULL);
 		if (date >= 0) {
 			xfer->current_time = date;
-			g_message("server date %"G_GUINT64_FORMAT, xfer->current_time);
+			g_message("nbd server received HTTP server date %"G_GUINT64_FORMAT, xfer->current_time);
 		}
 	} else if (g_str_equal(h_pair[0], "last-modified")) {
 		time_t date = curl_getdate(h_pair[1], NULL);
 		if (date >= 0) {
 			xfer->modified_time = date;
-			g_message("file date %"G_GUINT64_FORMAT, xfer->modified_time);
+			g_message("nbd server received HTTP file date %"G_GUINT64_FORMAT, xfer->modified_time);
 		}
 	}
 
@@ -526,7 +526,7 @@ static void start_read(struct RaucNBDContext *ctx, struct RaucNBDTransfer *xfer)
 		g_error("unexpected error from curl_multi_add_handle in %s", G_STRFUNC);
 }
 
-/* Appends Gstrv elements to curl_slist (only pointers, no data).
+/* Appends Gstrv elements to curl_slist (strings are copied).
  * If curl_slist does not exist yet (NULL passed), it will be created.
  * The created list needs to be freed (after usage) by the caller with
  * curl_slist_free_all(). */
@@ -556,13 +556,14 @@ static void start_configure(struct RaucNBDContext *ctx, struct RaucNBDTransfer *
 	gboolean res = FALSE;
 	CURLcode code = 0;
 	CURLMcode mcode = 0;
-	g_autofree guint8 *data = g_malloc(xfer->request.len);
-	g_autoptr(GVariant) v = NULL;
-	GVariantDict dict;
 
 	/* only read from the client on the first try */
 	if (!ctx->url) {
-		GStrv info_headers = NULL; /* array of strings such as 'Foo: bar' */
+		g_autofree guint8 *data = g_malloc(xfer->request.len);
+		g_autoptr(GVariant) v = NULL;
+		g_auto(GVariantDict) dict = G_VARIANT_DICT_INIT(NULL);
+		g_auto(GStrv) headers = NULL; /* array of strings such as 'Foo: bar' */
+		g_auto(GStrv) info_headers = NULL; /* array of strings such as 'Foo: bar' */
 
 		res = r_read_exact(ctx->sock, (guint8*)data, xfer->request.len, NULL);
 		g_assert_true(res);
@@ -574,30 +575,29 @@ static void start_configure(struct RaucNBDContext *ctx, struct RaucNBDTransfer *
 		g_assert_nonnull(v);
 		{
 			g_autofree gchar *tmp = g_variant_print(v, TRUE);
-			g_message("received: %s", tmp);
+			g_message("nbd server received configuration: %s", tmp);
 		}
 
 		g_variant_dict_init(&dict, v);
-
 		g_variant_dict_lookup(&dict, "url", "s", &ctx->url);
 		g_variant_dict_lookup(&dict, "cert", "s", &ctx->tls_cert);
 		g_variant_dict_lookup(&dict, "key", "s", &ctx->tls_key);
 		g_variant_dict_lookup(&dict, "ca", "s", &ctx->tls_ca);
 		g_variant_dict_lookup(&dict, "no-verify", "b", &ctx->tls_no_verify);
-		g_variant_dict_lookup(&dict, "headers", "^as", &ctx->headers);
+		g_variant_dict_lookup(&dict, "headers", "^as", &headers);
 		g_variant_dict_lookup(&dict, "info-headers", "^as", &info_headers);
 		g_assert_nonnull(ctx->url);
 
-		if (ctx->headers) {
-			ctx->headers_slist = gstrv_add_to_slist(NULL, ctx->headers);
-			ctx->initial_headers_slist = gstrv_add_to_slist(NULL, ctx->headers);
+		if (headers) {
+			ctx->headers_slist = gstrv_add_to_slist(NULL, headers);
+			ctx->initial_headers_slist = gstrv_add_to_slist(NULL, headers);
 		}
 		if (info_headers) {
 			ctx->initial_headers_slist = gstrv_add_to_slist(ctx->initial_headers_slist, info_headers);
 		}
 	}
 
-	g_message("configuring for URL: %s", ctx->url);
+	g_message("nbd server configuring for URL: %s", ctx->url);
 
 	prepare_curl(xfer);
 	if (ctx->initial_headers_slist) {
@@ -619,6 +619,8 @@ static void start_configure(struct RaucNBDContext *ctx, struct RaucNBDTransfer *
 	xfer->buffer_size = 4;
 	xfer->buffer_pos = 0;
 
+	g_debug("nbd server sending initial range request to HTTP server");
+
 	mcode = curl_multi_add_handle(ctx->multi, xfer->easy);
 	if (mcode != CURLM_OK)
 		g_error("unexpected error from curl_multi_add_handle in %s", G_STRFUNC);
@@ -632,8 +634,9 @@ static void start_request(struct RaucNBDContext *ctx, struct RaucNBDTransfer *xf
 			break;
 		}
 		case NBD_CMD_DISC: {
-			g_message("disconnect");
+			g_message("nbd server received disconnect request");
 			ctx->done = TRUE;
+			g_free(xfer); /* not queued via curl_multi_add_handle */
 			break;
 		}
 		case RAUC_NBD_CMD_CONFIGURE: {
@@ -641,7 +644,7 @@ static void start_request(struct RaucNBDContext *ctx, struct RaucNBDTransfer *xf
 			break;
 		}
 		default: {
-			g_error("bad request type");
+			g_error("nbd server received bad request type");
 			break;
 		}
 	}
@@ -694,6 +697,7 @@ static gboolean finish_configure(struct RaucNBDContext *ctx, struct RaucNBDTrans
 	gboolean res = FALSE;
 	CURLcode code;
 	long response_code = 0;
+	long http_version = 0;
 	const char *effective_url = NULL;
 	g_auto(GVariantDict) dict = G_VARIANT_DICT_INIT(NULL);
 	g_autoptr(GVariant) v = NULL;
@@ -745,11 +749,18 @@ static gboolean finish_configure(struct RaucNBDContext *ctx, struct RaucNBDTrans
 	code = curl_easy_getinfo(xfer->easy, CURLINFO_EFFECTIVE_URL, &effective_url);
 	if (code == CURLE_OK) {
 		if (!g_str_equal(ctx->url, effective_url))
-			g_message("redirected from %s to %s",ctx->url, effective_url);
+			g_message("redirected from %s to %s", ctx->url, effective_url);
 		g_free(ctx->url);
 		ctx->url = g_strdup(effective_url);
 	}
 
+	code = curl_easy_getinfo(xfer->easy, CURLINFO_HTTP_VERSION, &http_version);
+	if (code == CURLE_OK) {
+		if (http_version == CURL_HTTP_VERSION_1_0 ||
+		    http_version == CURL_HTTP_VERSION_1_1)
+			g_warning("using HTTP/1 for streaming, expect slow installation; enable HTTP/2 if possible");
+	}
+
 	collect_curl_stats(ctx, xfer);
 
 	res = TRUE;
@@ -828,7 +839,10 @@ gboolean r_nbd_run_server(gint sock, GError **error)
 		return FALSE;
 	}
 
-	g_message("running as UID %d, GID %d", getuid(), getgid());
+	// let us handle broken pipes explicitly
+	signal(SIGPIPE, SIG_IGN);
+
+	g_message("nbd server running as UID %d, GID %d", getuid(), getgid());
 
 	ctx.dl_size = r_stats_new("nbd dl_size");
 	ctx.dl_speed = r_stats_new("nbd dl_speed");
@@ -918,8 +932,8 @@ gboolean r_nbd_run_server(gint sock, GError **error)
 				xfer->reply.error = GUINT32_TO_BE(5); /* NBD_EIO */
 				xfer->done = TRUE;
 			} else {
-				g_message("request failed: %s (retrying)", curl_easy_strerror(msg->data.result));
 				xfer->errors++;
+				g_message("request failed: %s (retrying %d/5)", xfer->errbuf, xfer->errors);
 			}
 
 			res = finish_request(&ctx, xfer);
@@ -928,6 +942,7 @@ gboolean r_nbd_run_server(gint sock, GError **error)
 						error,
 						R_NBD_ERROR, R_NBD_ERROR_SHUTDOWN,
 						"finish_request failed, shutting down");
+				g_free(xfer);
 				goto out;
 			}
 
@@ -950,6 +965,10 @@ gboolean r_nbd_run_server(gint sock, GError **error)
 
 	res = TRUE;
 out:
+	g_clear_pointer(&ctx.url, g_free);
+	g_clear_pointer(&ctx.tls_cert, g_free);
+	g_clear_pointer(&ctx.tls_key, g_free);
+	g_clear_pointer(&ctx.tls_ca, g_free);
 	g_clear_pointer(&ctx.dl_size, r_stats_free);
 	g_clear_pointer(&ctx.dl_speed, r_stats_free);
 	g_clear_pointer(&ctx.namelookup, r_stats_free);
@@ -959,7 +978,7 @@ out:
 	curl_multi_cleanup(ctx.multi);
 	g_clear_pointer(&ctx.headers_slist, curl_slist_free_all);
 	g_clear_pointer(&ctx.initial_headers_slist, curl_slist_free_all);
-	g_message("exiting nbd server");
+	g_message("nbd server exiting");
 	return res;
 }
 
@@ -1003,7 +1022,7 @@ static gboolean nbd_configure(RaucNBDServer *nbd_srv, GError **error)
 	v = g_variant_dict_end(&dict);
 	{
 		g_autofree gchar *tmp = g_variant_print(v, TRUE);
-		g_message("sending: %s", tmp);
+		g_message("sending config request to nbd server: %s", tmp);
 	}
 
 	request.magic = GUINT32_TO_BE(NBD_REQUEST_MAGIC);
@@ -1068,11 +1087,22 @@ static gboolean nbd_configure(RaucNBDServer *nbd_srv, GError **error)
 
 	g_variant_dict_lookup(&dict, "url", "s", &nbd_srv->effective_url);
 	g_variant_dict_lookup(&dict, "size", "t", &nbd_srv->data_size);
-	g_message("received total size %"G_GUINT64_FORMAT, nbd_srv->data_size);
+	if (nbd_srv->data_size) {
+		g_autofree gchar* formatted_size = g_format_size_full(nbd_srv->data_size, G_FORMAT_SIZE_LONG_FORMAT);
+		g_message("received HTTP server info: total size %s", formatted_size);
+	}
 	g_variant_dict_lookup(&dict, "current-time", "t", &nbd_srv->current_time);
-	g_message("received current time %"G_GUINT64_FORMAT, nbd_srv->current_time);
+	if (nbd_srv->current_time) {
+		g_autoptr(GDateTime) datetime = g_date_time_new_from_unix_utc(nbd_srv->current_time);
+		g_autofree gchar *formatted_date = g_date_time_format(datetime, "%Y-%m-%d %H:%M:%S");
+		g_message("received HTTP server info: current time %s (%"G_GUINT64_FORMAT ")", formatted_date, nbd_srv->current_time);
+	}
 	g_variant_dict_lookup(&dict, "modified-time", "t", &nbd_srv->modified_time);
-	g_message("received modified time %"G_GUINT64_FORMAT, nbd_srv->modified_time);
+	if (nbd_srv->modified_time) {
+		g_autoptr(GDateTime) datetime = g_date_time_new_from_unix_utc(nbd_srv->modified_time);
+		g_autofree gchar *formatted_date = g_date_time_format(datetime, "%Y-%m-%d %H:%M:%S");
+		g_message("received HTTP server info: modified time %s (%"G_GUINT64_FORMAT ")", formatted_date, nbd_srv->modified_time);
+	}
 
 	return TRUE;
 }
@@ -1161,6 +1191,8 @@ gboolean r_nbd_start_server(RaucNBDServer *nbd_srv, GError **error)
 	g_return_val_if_fail(nbd_srv != NULL, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
+	g_message("starting the nbd server");
+
 	if (socketpair(AF_UNIX, SOCK_STREAM, 0, sockets) < 0) {
 		g_set_error(
 				error,
@@ -1224,7 +1256,7 @@ gboolean r_nbd_start_server(RaucNBDServer *nbd_srv, GError **error)
 		goto out;
 	}
 
-	g_message("nbd server started");
+	g_message("the nbd server was started");
 
 	res = TRUE;
 
@@ -1247,7 +1279,7 @@ gboolean r_nbd_stop_server(RaucNBDServer *nbd_srv, GError **error)
 	if (!nbd_srv->sproc)
 		return TRUE;
 
-	g_message("nbd server stopping");
+	g_message("stopping the nbd server");
 
 	if (nbd_srv->sock >= 0) {
 		struct nbd_request request = {0};
@@ -1257,8 +1289,10 @@ gboolean r_nbd_stop_server(RaucNBDServer *nbd_srv, GError **error)
 		request.magic = GUINT32_TO_BE(NBD_REQUEST_MAGIC);
 		request.type = GUINT32_TO_BE(NBD_CMD_DISC);
 		request.len = 0;
-		if (!r_write_exact(nbd_srv->sock, (guint8*)&request, sizeof(request), NULL))
-			g_error("failed to send nbd disconnect request");
+		if (!r_write_exact(nbd_srv->sock, (guint8*)&request, sizeof(request), &ierror)) {
+			g_message("failed to send nbd disconnect request, closing socket: %s", ierror->message);
+			g_clear_error(&ierror);
+		}
 
 		g_close(nbd_srv->sock, NULL);
 		nbd_srv->sock = -1;
@@ -1273,7 +1307,7 @@ gboolean r_nbd_stop_server(RaucNBDServer *nbd_srv, GError **error)
 		goto out;
 	}
 
-	g_message("nbd server stopped");
+	g_message("the nbd server was stopped");
 
 out:
 	g_clear_object(&nbd_srv->sproc);
