@@ -24,7 +24,7 @@ static gboolean service_install_notify(gpointer data)
 
 	g_mutex_lock(&args->status_mutex);
 	while (!g_queue_is_empty(&args->status_messages)) {
-		gchar *msg = g_queue_pop_head(&args->status_messages);
+		g_autofree gchar *msg = g_queue_pop_head(&args->status_messages);
 		g_message("installing %s: %s", args->name, msg);
 	}
 	g_mutex_unlock(&args->status_mutex);
@@ -104,6 +104,9 @@ static gboolean r_on_handle_install_bundle(
 	if (g_variant_dict_lookup(&dict, "ignore-compatible", "b", &args->ignore_compatible))
 		g_variant_dict_remove(&dict, "ignore-compatible");
 
+	if (g_variant_dict_lookup(&dict, "ignore-version-limit", "b", &args->ignore_version_limit))
+		g_variant_dict_remove(&dict, "ignore-version-limit");
+
 	if (g_variant_dict_lookup(&dict, "transaction-id", "s", &args->transaction))
 		g_variant_dict_remove(&dict, "transaction-id");
 
