@@ -69,7 +69,7 @@ static gboolean clear_slot(RaucSlot *slot, GError **error)
 	g_autoptr(GOutputStream) outstream = NULL;
 	gint write_count = 0;
 
-	outstream = G_OUTPUT_STREAM(r_open_unix_output_stream(slot->device, NULL, &ierror));
+	outstream = G_OUTPUT_STREAM(r_unix_output_stream_open_device(slot->device, NULL, &ierror));
 	if (outstream == NULL) {
 		g_propagate_error(error, ierror);
 		return FALSE;
@@ -476,8 +476,20 @@ static gboolean casync_extract(RaucImage *image, gchar *dest, int out_fd, const
 	g_ptr_array_add(args, NULL);
 
 	launcher = g_subprocess_launcher_new(G_SUBPROCESS_FLAGS_NONE);
-	if (out_fd >= 0)
-		g_subprocess_launcher_take_stdout_fd(launcher, out_fd);
+
+	if (out_fd >= 0) {
+		/* Must be a copy: glib automatically closes stdout_fd, but we close out_fd as well */
+		int out_fd_copy = dup(out_fd);
+		if (out_fd_copy == -1) {
+			g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
+					"Failed to dup() output file descriptor: %s",
+					strerror(errno));
+			res = FALSE;
+			goto out;
+		}
+		g_subprocess_launcher_take_stdout_fd(launcher, out_fd_copy);
+	}
+
 	if (tmpdir)
 		g_subprocess_launcher_setenv(launcher, "TMPDIR", tmpdir, TRUE);
 
@@ -638,7 +650,7 @@ static gboolean copy_raw_image_to_dev(RaucImage *image, RaucSlot *slot, GError *
 
 	/* open */
 	g_message("opening slot device %s", slot->device);
-	outstream = r_open_unix_output_stream(slot->device, NULL, &ierror);
+	outstream = r_unix_output_stream_open_device(slot->device, NULL, &ierror);
 	if (outstream == NULL) {
 		res = FALSE;
 		g_propagate_error(error, ierror);
@@ -952,6 +964,7 @@ static gboolean ubifs_format_slot(RaucSlot *dest_slot, GError **error)
 
 	g_ptr_array_add(args, g_strdup("mkfs.ubifs"));
 	g_ptr_array_add(args, g_strdup("-y"));
+	r_ptr_array_addv(args, dest_slot->extra_mkfs_opts, TRUE);
 	g_ptr_array_add(args, g_strdup(dest_slot->device));
 	g_ptr_array_add(args, NULL);
 
@@ -995,7 +1008,7 @@ static gboolean ext4_format_slot(RaucSlot *dest_slot, GError **error)
 {
 	GError *ierror = NULL;
 	gboolean res = FALSE;
-	g_autoptr(GPtrArray) args = g_ptr_array_new_full(4, g_free);
+	g_autoptr(GPtrArray) args = g_ptr_array_new_full(6, g_free);
 
 	g_ptr_array_add(args, g_strdup("mkfs.ext4"));
 	g_ptr_array_add(args, g_strdup("-F"));
@@ -1004,6 +1017,7 @@ static gboolean ext4_format_slot(RaucSlot *dest_slot, GError **error)
 		g_ptr_array_add(args, g_strdup(dest_slot->name));
 	}
 	g_ptr_array_add(args, g_strdup("-I256"));
+	r_ptr_array_addv(args, dest_slot->extra_mkfs_opts, TRUE);
 	g_ptr_array_add(args, g_strdup(dest_slot->device));
 	g_ptr_array_add(args, NULL);
 
@@ -1073,6 +1087,7 @@ static gboolean vfat_format_slot(RaucSlot *dest_slot, GError **error)
 	g_ptr_array_add(args, g_strdup("mkfs.vfat"));
 	g_ptr_array_add(args, g_strdup("-n"));
 	g_ptr_array_add(args, vfat_label_generator(dest_slot->name));
+	r_ptr_array_addv(args, dest_slot->extra_mkfs_opts, TRUE);
 	g_ptr_array_add(args, g_strdup(dest_slot->device));
 	g_ptr_array_add(args, NULL);
 
@@ -1338,15 +1353,10 @@ static gboolean run_slot_hook_extra_env(const gchar *hook_name, const gchar *hoo
 
 	bundle = r_context()->install_info->mounted_bundle;
 	if (bundle) {
-		gchar **hashes = NULL;
-		gchar *string = NULL;
-
-		hashes = get_pubkey_hashes(bundle->verified_chain);
-		string = g_strjoinv(" ", hashes);
-		g_strfreev(hashes);
+		g_auto(GStrv) hashes = get_pubkey_hashes(bundle->verified_chain);
+		g_autofree gchar *string = g_strjoinv(" ", hashes);
 
 		g_subprocess_launcher_setenv(launcher, "RAUC_BUNDLE_SPKI_HASHES", string, TRUE);
-		g_free(string);
 
 		g_subprocess_launcher_setenv(launcher, "RAUC_BUNDLE_MOUNT_POINT", bundle->mount_point, TRUE);
 	}
@@ -1466,7 +1476,7 @@ static gboolean img_to_ubivol_handler(RaucImage *image, RaucSlot *dest_slot, con
 
 	/* open */
 	g_message("opening slot device %s", dest_slot->device);
-	outstream = r_open_unix_output_stream(dest_slot->device, &out_fd, &ierror);
+	outstream = r_unix_output_stream_open_device(dest_slot->device, &out_fd, &ierror);
 	if (outstream == NULL) {
 		res = FALSE;
 		g_propagate_error(error, ierror);
@@ -1535,7 +1545,7 @@ static gboolean img_to_ubifs_handler(RaucImage *image, RaucSlot *dest_slot, cons
 
 	/* open */
 	g_message("opening slot device %s", dest_slot->device);
-	outstream = r_open_unix_output_stream(dest_slot->device, &out_fd, &ierror);
+	outstream = r_unix_output_stream_open_device(dest_slot->device, &out_fd, &ierror);
 	if (outstream == NULL) {
 		res = FALSE;
 		g_propagate_error(error, ierror);
@@ -2258,7 +2268,7 @@ static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot,
 
 	/* open */
 	g_message("Opening slot device partition %s", part_slot->device);
-	outstream = r_open_unix_output_stream(part_slot->device, &out_fd, &ierror);
+	outstream = r_unix_output_stream_open_device(part_slot->device, &out_fd, &ierror);
 	if (outstream == NULL) {
 		g_propagate_error(error, ierror);
 		res = FALSE;
@@ -2333,7 +2343,7 @@ static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot,
 
 	if (part_active == part_active_after) {
 		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
-				"Toggling the boot partition failed! Your kernel is most-likely affected by the ioctl ext_csd bug: see http://rauc.readthedocs.io/en/latest/advanced.html#update-emmc-boot-partitions");
+				"Toggling the boot partition failed! Your kernel is most-likely affected by the ioctl ext_csd bug: see https://rauc.readthedocs.io/en/latest/advanced.html#update-bootloader-in-emmc-boot-partitions");
 		res = FALSE;
 		goto out;
 	}
