@@ -52,10 +52,9 @@ typedef struct {
 
 static gchar *sfdisk_get(const gchar *device)
 {
-	GSubprocess *sub;
+	g_autoptr(GSubprocess) sub = NULL;
 	GError *error = NULL;
-	GBytes *stdout_buf = NULL;
-	gsize size;
+	g_autofree gchar *stdout_buf = NULL;
 	gboolean res = FALSE;
 
 	sub = g_subprocess_new(
@@ -68,7 +67,7 @@ static gchar *sfdisk_get(const gchar *device)
 	g_assert_no_error(error);
 	g_assert_nonnull(sub);
 
-	res = g_subprocess_communicate(sub, NULL, NULL, &stdout_buf, NULL, &error);
+	res = g_subprocess_communicate_utf8(sub, NULL, NULL, &stdout_buf, NULL, &error);
 	g_assert_no_error(error);
 	g_assert_true(res);
 
@@ -76,7 +75,7 @@ static gchar *sfdisk_get(const gchar *device)
 	g_assert_no_error(error);
 	g_assert_true(res);
 
-	return g_bytes_unref_to_data(stdout_buf, &size);
+	return g_steal_pointer(&stdout_buf);
 }
 
 static void sfdisk_check(const gchar *device, const gchar *expected)
@@ -129,7 +128,7 @@ static void sfdisk_check(const gchar *device, const gchar *expected)
 
 static void sfdisk_set(const gchar *device, const gchar *dump)
 {
-	GSubprocess *sub;
+	g_autoptr(GSubprocess) sub = NULL;
 	GError *error = NULL;
 	g_autoptr(GBytes) stdin_buf = NULL;
 	gboolean res = FALSE;
@@ -275,9 +274,13 @@ static void test_boot_switch(BootSwitchFixture *fixture,
 		gconstpointer user_data)
 {
 	BootSwitchData *data = (BootSwitchData*) user_data;
-	gchar *slotpath, *imagename, *imagepath, *mountprefix, *hookpath = NULL;
-	RaucImage *image;
-	RaucSlot *targetslot;
+	g_autofree gchar *slotpath = NULL;
+	g_autofree gchar *imagename = NULL;
+	g_autofree gchar *imagepath = NULL;
+	g_autofree gchar *mountprefix = NULL;
+	g_autofree gchar *hookpath = NULL;
+	RaucImage *image = NULL;
+	RaucSlot *targetslot = NULL;
 	img_to_slot_handler handler;
 	GError *ierror = NULL;
 	gboolean res = FALSE;
@@ -308,7 +311,7 @@ static void test_boot_switch(BootSwitchFixture *fixture,
 	clear_device(slotpath);
 
 	/* create source image */
-	image = g_new0(RaucImage, 1);
+	image = r_new_image();
 	image->slotclass = g_strdup("rootfs");
 	image->filename = g_strdup(imagepath);
 	image->checksum.size = IMAGE_SIZE;
@@ -366,6 +369,7 @@ static void test_boot_switch(BootSwitchFixture *fixture,
 
 	if (data->params & BOOT_SWITCH_EXPECT_FAIL) {
 		g_assert_error(ierror, data->err_domain, data->err_code);
+		g_clear_error(&ierror);
 		g_assert_false(res);
 		goto out;
 	} else {
@@ -395,11 +399,6 @@ out:
 
 	g_rmdir(mountprefix);
 
-	g_free(slotpath);
-	g_free(imagename);
-	g_free(imagepath);
-	g_clear_pointer(&hookpath, g_free);
-	g_free(mountprefix);
 	r_free_image(image);
 	r_slot_free(targetslot);
 }
