@@ -177,6 +177,66 @@ gboolean test_rm_tree(const gchar *dirname, const gchar *filename)
 	return res;
 }
 
+int test_prepare_system_conf(const gchar *dirname, const gchar *filename, const SystemTestOptions *options)
+{
+	g_autofree gchar *path = g_build_filename(dirname, filename, NULL);
+	g_autoptr(GString) config = g_string_sized_new(1024);
+
+	g_string_append(config, "[system]\n\
+compatible=Test Config\n\
+bootloader=grub\n\
+grubenv=grubenv.test\n\
+variant-name=Default Variant\n\
+");
+	if (options && options->min_bundle_version)
+		g_string_append_printf(config, "min-bundle-version=%s\n", options->min_bundle_version);
+	g_string_append(config, "\n");
+
+	g_string_append(config, "[handlers]\n\
+system-info=bin/systeminfo.sh\n\
+pre-install=bin/preinstall.sh\n\
+post-install=bin/postinstall.sh\n\
+\n\
+[keyring]\n\
+path=openssl-ca/dev-ca.pem\n\
+check-crl=true\n\
+\n\
+[slot.rescue.0]\n\
+device=images/rescue-0\n\
+type=ext4\n\
+bootname=factory0\n\
+readonly=true\n\
+\n\
+[slot.rootfs.0]\n\
+device=images/rootfs-0\n\
+type=ext4\n\
+bootname=system0\n\
+\n\
+[slot.rootfs.1]\n\
+device=images/rootfs-1\n\
+type=ext4\n\
+bootname=system1\n\
+\n\
+[slot.appfs.0]\n\
+device=images/appfs-0\n\
+type=ext4\n\
+parent=rootfs.0\n\
+\n\
+[slot.appfs.1]\n\
+device=images/appfs-1\n\
+type=ext4\n\
+parent=rootfs.1\n\
+\n\
+[slot.bootloader.0]\n\
+device=images/bootloader-0\n\
+type=ext4\n\
+allow-mounted=true\n\
+");
+
+	g_assert_true(g_file_set_contents(path, config->str, config->len, NULL));
+
+	return 0;
+}
 int test_prepare_manifest_file(const gchar *dirname, const gchar *filename, const ManifestTestOptions *options)
 {
 	g_autofree gchar *path = g_build_filename(dirname, filename, NULL);
@@ -184,7 +244,10 @@ int test_prepare_manifest_file(const gchar *dirname, const gchar *filename, cons
 	RaucImage *img;
 
 	rm->update_compatible = g_strdup("Test Config");
-	rm->update_version = g_strdup("2011.03-2");
+	if (options->bundle_version)
+		rm->update_version = g_strdup(options->bundle_version);
+	else if (!options->no_bundle_version)
+		rm->update_version = g_strdup("2011.03-2");
 
 	rm->bundle_format = options->format;
 
@@ -196,14 +259,14 @@ int test_prepare_manifest_file(const gchar *dirname, const gchar *filename, cons
 	}
 
 	if (options->slots) {
-		img = g_new0(RaucImage, 1);
+		img = r_new_image();
 		img->slotclass = g_strdup("rootfs");
 		img->filename = g_strdup("rootfs.ext4");
 		if (options->hooks)
 			img->hooks.post_install = TRUE;
 		rm->images = g_list_append(rm->images, img);
 
-		img = g_new0(RaucImage, 1);
+		img = r_new_image();
 		img->slotclass = g_strdup("appfs");
 		img->filename = g_strdup("appfs.ext4");
 		rm->images = g_list_append(rm->images, img);
@@ -418,8 +481,19 @@ void flip_bits_filename(gchar *filename, off_t offset, guint8 mask)
 
 void replace_strdup(gchar **dst, const gchar *src)
 {
-	g_free(*dst);
-	*dst = g_strdup(src);
+	r_replace_strdup(dst, src);
+}
+
+GPtrArray *test_ptr_array_from_strsplit(const gchar *input)
+{
+	GPtrArray *result = g_ptr_array_new_with_free_func(g_free);
+	/* The strings will be owned by the returned GPtrArray, so we don't
+	 * want to free them via g_auto(GStrv). */
+	g_autofree GStrv strv = g_strsplit(input, ";", 0);
+
+	r_ptr_array_addv(result, strv, FALSE);
+
+	return result;
 }
 
 void* dup_test_mem(GPtrArray *ptrs, const void *mem, gsize len)
