@@ -10,8 +10,14 @@ typedef struct {
 	gboolean hooks;
 	gboolean slots;
 	RManifestBundleFormat format;
+	gboolean no_bundle_version;
+	const gchar *bundle_version;
 } ManifestTestOptions;
 
+typedef struct {
+	const gchar *min_bundle_version;
+} SystemTestOptions;
+
 guint8* random_bytes(gsize size, guint32 seed);
 gchar* write_random_file(const gchar *tmpdir, const gchar *filename,
 		gsize size, const guint32 seed);
@@ -23,6 +29,7 @@ int test_rmdir(const gchar *dirname, const gchar *filename);
 int test_remove(const gchar *dirname, const gchar *filename);
 int test_lstat(const gchar *dirname, const gchar *filename, GStatBuf *buf);
 gboolean test_rm_tree(const gchar *dirname, const gchar *filename);
+int test_prepare_system_conf(const gchar *dirname, const gchar *filename, const SystemTestOptions *options);
 int test_prepare_manifest_file(const gchar *dirname, const gchar *filename, const ManifestTestOptions *options);
 gboolean test_make_filesystem(const gchar *dirname, const gchar *filename);
 gboolean test_mount(const gchar *src, const gchar *dest);
@@ -38,6 +45,12 @@ void flip_bits_fd(int fd, off_t offset, guint8 mask);
 void flip_bits_filename(gchar *filename, off_t offset, guint8 mask);
 void replace_strdup(gchar **dst, const gchar *src);
 
+/**
+ * Build a GPtrArray from a ;-separated string. The elements in the array are
+ * freed with the array itself.
+ */
+GPtrArray *test_ptr_array_from_strsplit(const gchar *input);
+
 void* dup_test_mem(GPtrArray *ptrs, const void *mem, gsize len);
 void* dup_test_printf(GPtrArray *ptrs, const gchar *str, ...)
 __attribute__((__format__(__printf__, 2, 3)));
