@@ -30,7 +30,7 @@ static void dm_fixture_set_up(DMFixture *fixture,
 {
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 	g_assert_nonnull(fixture->tmpdir);
-	g_print("dm tmpdir: %s\n", fixture->tmpdir);
+	g_test_message("dm tmpdir: %s\n", fixture->tmpdir);
 }
 
 static void dm_fixture_tear_down(DMFixture *fixture,
@@ -231,7 +231,7 @@ static void dm_verity_simple_test(void)
 	g_assert_no_error(error);
 	g_assert_true(res);
 
-	for (int i = 0; i<129; i++) {
+	for (int i = 0; i < 129; i++) {
 		int r = read(fd, buf, sizeof(buf));
 		g_assert_cmpint(r, ==, 4096);
 		g_assert_cmpint(buf[0], ==, 0);
