@@ -16,6 +16,9 @@ GMainLoop *r_loop = NULL;
 
 typedef struct {
 	const gchar **message_needles;
+	GQuark install_err_domain;
+	gint install_err_code;
+	SystemTestOptions system_test_options;
 	ManifestTestOptions manifest_test_options;
 } InstallData;
 
@@ -25,7 +28,7 @@ static void install_fixture_set_up_bundle(InstallFixture *fixture,
 	InstallData *data = (InstallData*) user_data;
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
-	fixture_helper_set_up_system(fixture->tmpdir, NULL);
+	fixture_helper_set_up_system(fixture->tmpdir, NULL, &data->system_test_options);
 	fixture_helper_set_up_bundle(fixture->tmpdir, NULL, &data->manifest_test_options);
 }
 
@@ -35,7 +38,7 @@ static void install_fixture_set_up_bundle_central_status(InstallFixture *fixture
 	InstallData *data = (InstallData*) user_data;
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
-	fixture_helper_set_up_system(fixture->tmpdir, "test/test-global.conf");
+	fixture_helper_set_up_system(fixture->tmpdir, "test/test-global.conf", NULL);
 	fixture_helper_set_up_bundle(fixture->tmpdir, NULL, &data->manifest_test_options);
 }
 
@@ -57,7 +60,7 @@ hooks=install\n\
 
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
-	fixture_helper_set_up_system(fixture->tmpdir, NULL);
+	fixture_helper_set_up_system(fixture->tmpdir, NULL, NULL);
 	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, &data->manifest_test_options);
 }
 
@@ -79,7 +82,7 @@ filename=rootfs.ext4\n\
 
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
-	fixture_helper_set_up_system(fixture->tmpdir, NULL);
+	fixture_helper_set_up_system(fixture->tmpdir, NULL, NULL);
 	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, &data->manifest_test_options);
 }
 
@@ -103,7 +106,7 @@ filename=appfs.ext4";
 
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
-	fixture_helper_set_up_system(fixture->tmpdir, NULL);
+	fixture_helper_set_up_system(fixture->tmpdir, NULL, NULL);
 	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, &data->manifest_test_options);
 }
 
@@ -129,7 +132,7 @@ hooks=install";
 
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
-	fixture_helper_set_up_system(fixture->tmpdir, NULL);
+	fixture_helper_set_up_system(fixture->tmpdir, NULL, NULL);
 	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, &data->manifest_test_options);
 }
 
@@ -154,7 +157,7 @@ hooks=post-install";
 
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
-	fixture_helper_set_up_system(fixture->tmpdir, NULL);
+	fixture_helper_set_up_system(fixture->tmpdir, NULL, NULL);
 	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, &data->manifest_test_options);
 }
 
@@ -172,7 +175,24 @@ adaptive=adaptive-test-method;block-hash-index";
 
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
-	fixture_helper_set_up_system(fixture->tmpdir, "test/test-datadir.conf");
+	fixture_helper_set_up_system(fixture->tmpdir, "test/test-datadir.conf", NULL);
+	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, &data->manifest_test_options);
+}
+
+static void install_fixture_set_up_slot_skipping(InstallFixture *fixture,
+		gconstpointer user_data)
+{
+	InstallData *data = (InstallData*) user_data;
+	const gchar *manifest_file = "\
+[update]\n\
+compatible=Test Config\n\
+\n\
+[image.rootfs]\n\
+filename=rootfs.ext4";
+
+	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
+
+	fixture_helper_set_up_system(fixture->tmpdir, "test/test-install-same-false.conf", NULL);
 	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, &data->manifest_test_options);
 }
 
@@ -241,7 +261,7 @@ device=/path/to/prebootloader";
 
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 	g_assert_nonnull(fixture->tmpdir);
-	g_print("system conf tmpdir: %s\n", fixture->tmpdir);
+	g_test_message("system conf tmpdir: %s\n", fixture->tmpdir);
 
 	pathname = write_tmp_file(fixture->tmpdir, "system.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
@@ -1079,6 +1099,8 @@ static void install_test_bundle(InstallFixture *fixture,
 	g_autoptr(GError) ierror = NULL;
 	gboolean res;
 
+	g_assert_nonnull(data);
+
 	/* needs to run as root */
 	if (!test_running_as_root())
 		return;
@@ -1101,18 +1123,25 @@ static void install_test_bundle(InstallFixture *fixture,
 	args->notify = install_notify;
 	args->cleanup = install_cleanup;
 	res = do_install_bundle(args, &ierror);
-	g_assert_no_error(ierror);
-	g_assert_true(res);
+	if (!data->install_err_domain) {
+		g_assert_no_error(ierror);
+		g_assert_true(res);
 
-	slotfile = g_build_filename(fixture->tmpdir, "images/rootfs-1", NULL);
-	mountdir = g_build_filename(fixture->tmpdir, "mnt", NULL);
-	g_assert(test_mkdir_relative(fixture->tmpdir, "mnt", 0777) == 0);
-	testfilepath = g_build_filename(mountdir, "verify.txt", NULL);
-	g_assert(test_mount(slotfile, mountdir));
-	g_assert(g_file_test(testfilepath, G_FILE_TEST_IS_REGULAR));
-	g_assert(test_umount(fixture->tmpdir, "mnt"));
+		slotfile = g_build_filename(fixture->tmpdir, "images/rootfs-1", NULL);
+		mountdir = g_build_filename(fixture->tmpdir, "mnt", NULL);
+		g_assert(test_mkdir_relative(fixture->tmpdir, "mnt", 0777) == 0);
+		testfilepath = g_build_filename(mountdir, "verify.txt", NULL);
+		g_assert(test_mount(slotfile, mountdir));
+		g_assert(g_file_test(testfilepath, G_FILE_TEST_IS_REGULAR));
+		g_assert(test_umount(fixture->tmpdir, "mnt"));
+		g_assert(test_rm_tree(fixture->tmpdir, "mnt"));
+	} else {
+		/* expected installation error */
+		g_assert_error(ierror, data->install_err_domain, data->install_err_code);
+		g_assert_false(res);
+	}
 
-	if (data != NULL && data->message_needles != NULL) {
+	if (data->message_needles != NULL) {
 		const gchar **message_needles = data->message_needles;
 		while (*message_needles != NULL) {
 			g_assert_true(install_args_find_message(args, *message_needles));
@@ -1124,6 +1153,16 @@ static void install_test_bundle(InstallFixture *fixture,
 	args->cleanup(args);
 }
 
+static void install_test_bundle_twice(InstallFixture *fixture,
+		gconstpointer user_data)
+{
+	/* First run */
+	install_test_bundle(fixture, user_data);
+
+	/* Second run */
+	install_test_bundle(fixture, user_data);
+}
+
 static void install_test_bundle_thread(InstallFixture *fixture,
 		gconstpointer user_data)
 {
@@ -1433,7 +1472,7 @@ static void install_fixture_set_up_system_user(InstallFixture *fixture,
 {
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
-	fixture_helper_fixture_set_up_system_user(fixture->tmpdir, NULL);
+	fixture_helper_fixture_set_up_system_user(fixture->tmpdir, NULL, NULL);
 }
 
 int main(int argc, char *argv[])
@@ -1593,5 +1632,78 @@ int main(int argc, char *argv[])
 			install_fixture_set_up_bundle_adaptive, install_test_bundle,
 			install_fixture_tear_down);
 
+	g_test_add("/install/slot-skipping",
+			InstallFixture, install_data,
+			install_fixture_set_up_slot_skipping, install_test_bundle_twice,
+			install_fixture_tear_down);
+
+	install_data = dup_test_data(ptrs, (&(InstallData) {
+		.install_err_domain = R_INSTALL_ERROR,
+		.install_err_code = R_INSTALL_ERROR_VERSION_MISMATCH,
+		.system_test_options = {
+		        .min_bundle_version = "1.0",
+		},
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_VERITY,
+		        .slots = TRUE,
+		        .bundle_version = "0.0.1",
+		},
+	}));
+	g_test_add("/install/min-bundle-version/version-bad",
+			InstallFixture, install_data,
+			install_fixture_set_up_bundle,
+			install_test_bundle,
+			install_fixture_tear_down);
+
+	install_data = dup_test_data(ptrs, (&(InstallData) {
+		.system_test_options = {
+		        .min_bundle_version = "1.0",
+		},
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_VERITY,
+		        .slots = TRUE,
+		        .bundle_version = "1.0.1",
+		},
+	}));
+	g_test_add("/install/min-bundle-version/version-good-1",
+			InstallFixture, install_data,
+			install_fixture_set_up_bundle,
+			install_test_bundle,
+			install_fixture_tear_down);
+
+	install_data = dup_test_data(ptrs, (&(InstallData) {
+		.system_test_options = {
+		        .min_bundle_version = "1.0",
+		},
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_VERITY,
+		        .slots = TRUE,
+		        .bundle_version = "2025.1-foo+bar",
+		},
+	}));
+	g_test_add("/install/min-bundle-version/version-good-2",
+			InstallFixture, install_data,
+			install_fixture_set_up_bundle,
+			install_test_bundle,
+			install_fixture_tear_down);
+
+	install_data = dup_test_data(ptrs, (&(InstallData) {
+		.install_err_domain = R_INSTALL_ERROR,
+		.install_err_code = R_INSTALL_ERROR_VERSION_MISMATCH,
+		.system_test_options = {
+		        .min_bundle_version = "1.0",
+		},
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_VERITY,
+		        .slots = TRUE,
+		        .no_bundle_version = TRUE,
+		},
+	}));
+	g_test_add("/install/min-bundle-version/version-missing",
+			InstallFixture, install_data,
+			install_fixture_set_up_bundle,
+			install_test_bundle,
+			install_fixture_tear_down);
+
 	return g_test_run();
 }
