@@ -31,7 +31,7 @@ static void service_fixture_set_up(ServiceFixture *fixture, gconstpointer user_d
 
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
-	fixture_helper_set_up_system(fixture->tmpdir, NULL);
+	fixture_helper_set_up_system(fixture->tmpdir, NULL, NULL);
 
 	/* Write a D-Bus service file with current tmpdir */
 	contents = g_strdup_printf("\
@@ -73,6 +73,8 @@ static void service_fixture_tear_down(ServiceFixture *fixture, gconstpointer use
 	g_assert_no_error(error);
 	g_assert_true(ret);
 	g_free(fixture->tmpdir);
+
+	g_clear_pointer(&testloop, g_main_loop_unref);
 }
 
 static void on_installer_changed(GDBusProxy *proxy, GVariant *changed,
@@ -186,12 +188,12 @@ static void service_test_install(ServiceFixture *fixture, gconstpointer user_dat
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  40, "Updating slots", 2));
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  40, "Checking slot rootfs.1", 3));
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  43, "Checking slot rootfs.1 done.", 3));
-	for (gint32 i = 43; i<= 69; i++)
+	for (gint32 i = 43; i <= 69; i++)
 		g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)", i, "Copying image to rootfs.1", 3));
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  70, "Copying image to rootfs.1 done.", 3));
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  70, "Checking slot appfs.1", 3));
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  73, "Checking slot appfs.1 done.", 3));
-	for (gint32 i = 73; i<= 99; i++)
+	for (gint32 i = 73; i <= 99; i++)
 		g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)", i, "Copying image to appfs.1", 3));
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  99, "Copying image to appfs.1 done.", 3));
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  99, "Updating slots done.", 2));
