@@ -54,7 +54,7 @@ static void test_get_update_handler(UpdateHandlerFixture *fixture, gconstpointer
 	UpdateHandlerTestPair *test_pair = (UpdateHandlerTestPair*) user_data;
 	GError *ierror = NULL;
 
-	image = g_new0(RaucImage, 1);
+	image = r_new_image();
 	image->slotclass = g_strdup("rootfs");
 	image->filename = g_strconcat("rootfs.", test_pair->imagetype, NULL);
 
@@ -87,7 +87,7 @@ static void test_get_custom_update_handler(UpdateHandlerFixture *fixture, gconst
 	img_to_slot_handler handler;
 	GError *ierror = NULL;
 
-	image = g_new0(RaucImage, 1);
+	image = r_new_image();
 	image->slotclass = g_strdup("rootfs");
 	image->filename = g_strdup("rootfs.custom");
 	image->hooks.install = TRUE;
@@ -130,8 +130,8 @@ static void update_handler_fixture_set_up(UpdateHandlerFixture *fixture,
 				"Image detected as type: *");
 		g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_INFO,
 				"Selected adaptive update method *");
-		g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_INFO,
-				"building*");
+		g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
+				"Building new hash index for *");
 		g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
 				"Continuing after adaptive mode error: Slot * is too small for image *");
 	}
@@ -325,7 +325,11 @@ static void test_update_handler(UpdateHandlerFixture *fixture,
 		gconstpointer user_data)
 {
 	UpdateHandlerTestPair *test_pair = (UpdateHandlerTestPair*) user_data;
-	gchar *slotpath, *imagename, *imagepath, *mountprefix, *hookpath = NULL;
+	g_autofree gchar *slotpath = NULL;
+	g_autofree gchar *imagename = NULL;
+	g_autofree gchar *imagepath = NULL;
+	g_autofree gchar *mountprefix = NULL;
+	g_autofree gchar *hookpath = NULL;
 	goffset image_size;
 	RaucImage *image;
 	RaucSlot *targetslot;
@@ -342,7 +346,7 @@ static void test_update_handler(UpdateHandlerFixture *fixture,
 	/* prepare image and slot information */
 	image_size = IMAGE_SIZE;
 	imagename = g_strconcat("image.", test_pair->imagetype, NULL);
-	if (g_strcmp0(test_pair->slottype, "ubivol") == 0||
+	if (g_strcmp0(test_pair->slottype, "ubivol") == 0 ||
 	    g_strcmp0(test_pair->slottype, "ubifs") == 0) {
 		slotpath = g_strdup(g_getenv("RAUC_TEST_MTD_UBIVOL"));
 		if (!slotpath) {
@@ -389,7 +393,7 @@ static void test_update_handler(UpdateHandlerFixture *fixture,
 	}
 
 	/* create source image */
-	image = g_new0(RaucImage, 1);
+	image = r_new_image();
 	image->slotclass = g_strdup("rootfs");
 	image->filename = g_strdup(imagepath);
 	image->checksum.size = image_size;
@@ -493,11 +497,10 @@ no_image:
 		g_assert(test_mount(slotpath, mountprefix));
 		g_assert(r_umount(slotpath, NULL));
 	} else if ((g_strcmp0(test_pair->imagetype, "tar") == 0) || ((g_strcmp0(test_pair->imagetype, "caidx") == 0))) {
-		gchar *testpath = g_build_filename(mountprefix, "testfile.txt", NULL);
+		g_autofree gchar *testpath = g_build_filename(mountprefix, "testfile.txt", NULL);
 		g_assert(test_mount(slotpath, mountprefix));
 		g_assert_true(g_file_test(testpath, G_FILE_TEST_IS_REGULAR));
 		g_assert(r_umount(slotpath, NULL));
-		g_free(testpath);
 	}
 
 	/* check statistics */
@@ -598,13 +601,8 @@ out:
 		g_assert(g_remove(hookpath) == 0);
 	}
 
-	g_rmdir(mountprefix);
+	g_assert(g_rmdir(mountprefix) == 0);
 
-	g_free(slotpath);
-	g_free(imagename);
-	g_free(imagepath);
-	g_clear_pointer(&hookpath, g_free);
-	g_free(mountprefix);
 	r_free_image(image);
 	r_slot_free(targetslot);
 }
